@@ -12,6 +12,8 @@
 #include "options.h"
 #include "drive.h"
 #include "transport.h"
+#include "init.h"
+#include "write.h"
 
 /* ts A61007 */
 /* #include <a ssert.h> */
@@ -27,7 +29,7 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 {
 	struct burn_write_opts *opts;
 
-	opts = malloc(sizeof(struct burn_write_opts));
+	opts = calloc(1, sizeof(struct burn_write_opts));
 	if (opts == NULL) {
 		libdax_msgs_submit(libdax_messenger, -1, 0x00020111,
 			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
@@ -51,6 +53,9 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 	opts->do_stream_recording = 0;
 	opts->dvd_obs_override = 0;
 	opts->stdio_fsync_size = Libburn_stdio_fsync_limiT;
+	opts->text_packs = NULL;
+	opts->num_text_packs = 0;
+	opts->no_text_pack_crc_check = 0;
 	opts->has_mediacatalog = 0;
 	opts->format = BURN_CDROM;
 	opts->multi = 0;
@@ -60,15 +65,18 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 
 void burn_write_opts_free(struct burn_write_opts *opts)
 {
-	if (--opts->refcount <= 0)
-		free(opts);
+	if (--opts->refcount > 0)
+		return;
+	if (opts->text_packs != NULL)
+		free(opts->text_packs);
+	free(opts);
 }
 
 struct burn_read_opts *burn_read_opts_new(struct burn_drive *drive)
 {
 	struct burn_read_opts *opts;
 
-	opts = malloc(sizeof(struct burn_read_opts));
+	opts = calloc(1, sizeof(struct burn_read_opts));
 	opts->drive = drive;
 	opts->refcount = 1;
 	opts->raw = 0;
@@ -125,7 +133,7 @@ void burn_write_opts_set_toc_entries(struct burn_write_opts *opts, int count,
 				     struct burn_toc_entry *toc_entries)
 {
 	opts->toc_entries = count;
-	opts->toc_entry = malloc(count * sizeof(struct burn_toc_entry));
+	opts->toc_entry = calloc(count, sizeof(struct burn_toc_entry));
 	memcpy(opts->toc_entry, &toc_entries,
 	       sizeof(struct burn_toc_entry) * count);
 }
@@ -178,7 +186,7 @@ void burn_write_opts_set_has_mediacatalog(struct burn_write_opts *opts,
 void burn_write_opts_set_mediacatalog(struct burn_write_opts *opts,
 				      unsigned char mediacatalog[13])
 {
-	memcpy(opts->mediacatalog, &mediacatalog, 13);
+	memcpy(opts->mediacatalog, mediacatalog, 13);
 }
 
 
@@ -189,6 +197,64 @@ void burn_write_opts_set_multi(struct burn_write_opts *opts, int multi)
 }
 
 
+/* ts B11204 */
+/* @param flag bit0=do not verify checksums
+               bit1= repair mismatching checksums
+               bit2= repair checksums if they are 00 00 with each pack
+*/
+int burn_write_opts_set_leadin_text(struct burn_write_opts *opts,
+                                    unsigned char *text_packs,
+                                    int num_packs, int flag)
+{
+	int ret;
+	unsigned char *pack_buffer = NULL;
+
+	if (num_packs > Libburn_leadin_cdtext_packs_maX ) {
+		libdax_msgs_submit(libdax_messenger, opts->drive->global_index,
+				0x0002018b,
+				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+				"Too many CD-TEXT packs", 0, 0);
+		ret= 0; goto ex;
+	}
+
+	if (num_packs > 0)
+		BURN_ALLOC_MEM(pack_buffer, unsigned char, num_packs * 18);
+	
+	if (opts->text_packs != NULL) {
+		free(opts->text_packs);
+		opts->text_packs = NULL;
+	}
+
+	if (flag & 1) {
+		opts->no_text_pack_crc_check = 1;
+	} else {
+		opts->no_text_pack_crc_check = 0;
+		ret = burn_cdtext_crc_mismatches(text_packs, num_packs,
+							(flag >> 1) & 3);
+		if (ret > 0) {
+			libdax_msgs_submit(libdax_messenger, -1, 0x0002018f,
+				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+				"CD-TEXT pack CRC mismatch", 0, 0);
+			ret = 0; goto ex;
+		} else if (ret < 0) {
+			libdax_msgs_submit(libdax_messenger, -1, 0x00020190,
+				LIBDAX_MSGS_SEV_WARNING, LIBDAX_MSGS_PRIO_HIGH,
+			    "CD-TEXT pack CRC mismatch had to be corrected",
+				0, 0);
+		}
+	}
+
+	if (num_packs > 0) {
+		memcpy(pack_buffer, text_packs, num_packs * 18);
+		opts->text_packs = pack_buffer;
+	}
+	opts->num_text_packs = num_packs;
+	ret = 1;
+ex:;
+	return ret;
+}
+
+
 /* ts A61222 */
 void burn_write_opts_set_start_byte(struct burn_write_opts *opts, off_t value)
 {
@@ -296,6 +362,10 @@ do_sao:;
 	{wt = BURN_WRITE_SAO; goto ex;}
 no_sao:;
 try_tao:;
+	if (opts->num_text_packs > 0) {
+		strcat(reasons, "CD-TEXT: write type SAO required, ");
+		{wt = BURN_WRITE_NONE; goto ex;}
+	}
 	if ((flag & 1) && opts->write_type != BURN_WRITE_TAO)
 		goto try_raw;
 	reason_pt = reasons + strlen(reasons);
