@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -64,6 +64,7 @@ struct command
 	int error;
 	int retry;
 	struct buffer *page;
+	int timeout; /* milliseconds */
 };
 
 struct burn_scsi_inquiry_data
@@ -380,8 +381,9 @@ struct burn_drive
 	void (*send_parameters) (struct burn_drive *,
 				 const struct burn_read_opts *);
 	void (*send_write_parameters) (struct burn_drive *,
+				       struct burn_session *, int tno,
 				       const struct burn_write_opts *);
-	void (*send_cue_sheet) (struct burn_drive *, struct cue_sheet *);
+	int (*send_cue_sheet) (struct burn_drive *, struct cue_sheet *);
 
 	/* ts A70205 : Announce size of a DVD-R[W] DAO session. */
 	int (*reserve_track) (struct burn_drive *d, off_t size);
