@@ -141,6 +141,9 @@ struct isoburn {
  /* Libisofs image context */
  IsoImage *image;
 
+ /* The start LBA of the image */
+ int image_start_lba;
+
  /* The block data source from which the existing image is read.
  */
  IsoDataSource *iso_data_source;
@@ -260,7 +263,19 @@ int isoburn_get_msc2(struct isoburn *o,
 */
 IsoDataSource *
 isoburn_data_source_new(struct burn_drive *d,
-                         uint32_t displacement, int displacement_sign);
+                         uint32_t displacement, int displacement_sign,
+                         int cache_tiles, int tile_blocks);
+
+/** Default settings for above cache_tiles, tile_blocks in newly created
+    struct isoburn_read_opts.
+*/
+#define Libisoburn_default_cache_tileS 32
+#define Libisoburn_default_tile_blockS 32
+
+/** Maximum size of the cache in 2 kB blocks (1 GB)
+*/
+#define Libisoburn_cache_max_sizE (1024 * 512)
+
 
 /** Disable read capabilities of a data source which was originally created
     by isoburn_data_source_new(). After this any attempt to read will yield
@@ -282,6 +297,12 @@ int isoburn_data_source_shutdown(IsoDataSource *src, int flag);
 int isoburn_adjust_target_iso_head(struct isoburn *o,
                                    uint32_t offst, int flag);
 
+
+/** Initialize the root directory attributes of a freshly created image.
+*/
+int isoburn_root_defaults(IsoImage *image, int flag);
+
+
 /**
  * Options for image reading.
    (Comments here may be outdated. API getter/setter function descriptions
@@ -289,6 +310,9 @@ int isoburn_adjust_target_iso_head(struct isoburn *o,
     minor correction only.)
  */
 struct isoburn_read_opts {
+    int cache_tiles; /* number of cache tiles */
+    int cache_tile_blocks;
+
     unsigned int norock:1; /*< Do not read Rock Ridge extensions */
     unsigned int nojoliet:1; /*< Do not read Joliet extensions */
     unsigned int noiso1999:1; /*< Do not read ISO 9660:1999 enhanced tree */
@@ -434,6 +458,21 @@ struct isoburn_imgen_opts {
      */
     unsigned int allow_deep_paths :1;
 
+    /**
+     * If not allow_deep_paths is in effect, then it may become
+     * necessary to relocate directories so that no ECMA-119 file path
+     * has more than 8 components. These directories are grafted into either
+     * the root directory of the ISO image or into a dedicated relocation
+     * directory. For details see libisofs.h, iso_write_opts_set_rr_reloc().
+     */
+    char *rr_reloc_dir;       /* IsoNode name in root directory. NULL or
+                                 empty text means root itself. */
+    int rr_reloc_flags;       /* bit0= mark auto-created rr_reloc_dir by RE
+                                 bit1= not settable via API (used internally)
+                              */
+
+
+
     /**
      * Allow path in the ISO-9660 tree to have more than 255 characters.
      */
@@ -468,6 +507,13 @@ struct isoburn_imgen_opts {
      */
     unsigned int allow_full_ascii :1;
 
+    /**
+     * Like allow_full_ascii, but only allowing 7-bit characters.
+     * Lowercase letters get mapped to uppercase if not allow_lowercase is set.
+     * Gets overridden if allow_full_ascii is enabled.
+     */
+    unsigned int allow_7bit_ascii :1;
+
     /**
      * Allow paths in the Joliet tree to have more than 240 characters.
      */
