@@ -105,12 +105,38 @@ IsoImage *isoburn_get_attached_image(struct burn_drive *d)
 }
 
 
+/* API */
+int isoburn_get_attached_start_lba(struct burn_drive *d)
+{
+ int ret;
+ struct isoburn *o= NULL;
+
+ ret = isoburn_find_emulator(&o, d, 0);
+ if (ret < 0 || o == NULL)
+   return -1;
+ if(o->image == NULL)
+   return -1;
+ return o->image_start_lba;
+}
+
+
 static void isoburn_idle_free_function(void *ignored)
 {
  return;
 }
 
 
+int isoburn_root_defaults(IsoImage *image, int flag)
+{
+ IsoNode *root_node;
+ mode_t root_mode= 0755;
+
+ root_node= (IsoNode *) iso_image_get_root(image);
+ iso_node_set_permissions(root_node, root_mode);
+ return(1);
+}
+
+
 /* API function. See libisoburn.h
 */
 int isoburn_read_image(struct burn_drive *d,
@@ -133,6 +159,7 @@ int isoburn_read_image(struct burn_drive *d,
    if (ret < 0 || o == NULL)
      {ret= 0; goto ex;}
    status = isoburn_disc_get_status(d);
+   o->image_start_lba= -1;
  }
  if(read_opts==NULL) {
    isoburn_msgs_submit(o, 0x00060000,
@@ -176,6 +203,10 @@ create_blank_image:;
      }
      iso_image_set_ignore_aclea(o->image,
                          (!!(read_opts->noacl)) | ((!!read_opts->noea) << 1) );
+
+     ret= isoburn_root_defaults(o->image, 0);
+     if(ret <= 0)
+       goto ex;
    }
    {ret= 1; goto ex;}
  }
@@ -187,12 +218,12 @@ create_blank_image:;
    {ret= -4; goto ex;}
  }
  
- memset((char *) &ropts, 0, sizeof(ropts));
-
  ret = isoburn_disc_get_msc1(d, &int_num);
  if (ret <= 0)
    {ret= -2; goto ex;}
  ms_block= int_num;
+ if (o != NULL)
+   o->image_start_lba= ms_block;
  ret = isoburn_read_iso_head(d, int_num, &dummy, NULL, 0);
  if (ret <= 0) {
    sprintf(msg, "No ISO 9660 image at LBA %d. Creating blank image.", int_num);
@@ -251,11 +282,12 @@ displacement_rollover:;
  iso_read_opts_load_system_area(ropts, 1);
 
  ds = isoburn_data_source_new(d, read_opts->displacement,
-                              read_opts->displacement_sign);
+                         read_opts->displacement_sign,
+                         read_opts->cache_tiles, read_opts->cache_tile_blocks);
  if (ds == NULL) {
    isoburn_report_iso_error(ret, "Cannot create IsoDataSource object", 0,
                             "FATAL", 0);
-   goto ex;
+   ret= -1; goto ex;
  }
  if(o->iso_data_source!=NULL)
    iso_data_source_unref(o->iso_data_source);
@@ -269,6 +301,7 @@ displacement_rollover:;
  ret = iso_image_import(o->image, ds, ropts, &features);
  iso_tree_set_report_callback(o->image, NULL);
  iso_read_opts_free(ropts);
+ ropts= NULL;
 
  if (ret < 0) {
    isoburn_report_iso_error(ret, "Cannot import image", 0, "FAILURE", 0);
@@ -289,6 +322,8 @@ displacement_rollover:;
 ex:;
  if(msg != NULL)
    free(msg);
+ if(ropts != NULL)
+   iso_read_opts_free(ropts);
  return(ret);
 }
 
@@ -312,10 +347,29 @@ int isoburn_attach_image(struct burn_drive *d, IsoImage *image)
  if(o->image != NULL)
    iso_image_unref(o->image);
  o->image = image;
+ o->image_start_lba = -1;
  return(1);
 }
 
 
+/* API */
+int isoburn_attach_start_lba(struct burn_drive *d, int lba, int flag)
+{
+ int ret;
+ struct isoburn *o;
+
+ ret = isoburn_find_emulator(&o, d, 0);
+ if(ret < 0)
+   return ret;
+ if(o == NULL)
+   return 0;
+ if(o->image == NULL)
+   return 0;
+ o->image_start_lba = lba;
+ return 1;
+}
+
+
 /* API function. See libisoburn.h
 */
 int isoburn_activate_session(struct burn_drive *drive)
