@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
  * Copyright (c) 2007 Mario Danic
- * Copyright (c) 2009 - 2011 Thomas Schmitt
+ * Copyright (c) 2009 - 2012 Thomas Schmitt
  * 
  * This file is part of the libisofs project; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License version 2 
@@ -685,8 +685,37 @@ static int valid_j_char(uint16_t c)
         && cmp_ucsbe(&c, '\\');
 }
 
+/* @param relaxed bit0+1  0= strict ECMA-119
+                          1= additionally allow lowercase (else map to upper)
+                          2= allow all 8-bit characters
+                  bit2    allow all 7-bit characters (but map to upper if
+                          not bit0+1 == 2)
+*/
+static char map_fileid_char(char c, int relaxed)
+{
+    char upper;
+
+    if (c == '/')  /* Allowing slashes would cause lots of confusion */
+        return '_';
+    if ((relaxed & 3) == 2)
+        return c;
+    if (valid_d_char(c))
+        return c;
+    if ((relaxed & 4) && (c & 0x7f) == c && (c < 'a' || c > 'z'))
+        return c; 
+    upper= toupper(c);
+    if (valid_d_char(upper)) {
+        if (relaxed & 3) {
+            /* lower chars are allowed */
+            return c;
+        }
+        return upper;
+    }
+    return '_';
+}
+
 static
-char *iso_dirid(const char *src, int size)
+char *iso_dirid(const char *src, int size, int relaxed)
 {
     size_t len, i;
     char name[32];
@@ -696,25 +725,35 @@ char *iso_dirid(const char *src, int size)
         len = size;
     }
     for (i = 0; i < len; i++) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
         char c= toupper(src[i]);
         name[i] = valid_d_char(c) ? c : '_';
+
+#else /* Libisofs_old_ecma119_nameS */
+
+        name[i] = map_fileid_char(src[i], relaxed);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
     }
 
     name[len] = '\0';
     return strdup(name);
 }
 
-char *iso_1_dirid(const char *src)
+char *iso_1_dirid(const char *src, int relaxed)
 {
-    return iso_dirid(src, 8);
+    return iso_dirid(src, 8, relaxed);
 }
 
 char *iso_2_dirid(const char *src)
 {
-    return iso_dirid(src, 31);
+    return iso_dirid(src, 31, 0);
 }
 
-char *iso_1_fileid(const char *src)
+char *iso_1_fileid(const char *src, int relaxed, int force_dots)
 {
     char *dot; /* Position of the last dot in the filename, will be used 
                 * to calculate lname and lext. */
@@ -725,7 +764,8 @@ char *iso_1_fileid(const char *src)
         return NULL;
     }
     dot = strrchr(src, '.');
-
+    if (dot == src && strlen(src) > 4)
+        dot = NULL;      /* Use the long extension instead of the empty name */
     lext = dot ? strlen(dot + 1) : 0;
     lname = strlen(src) - lext - (dot ? 1 : 0);
 
@@ -738,19 +778,43 @@ char *iso_1_fileid(const char *src)
 
     /* Convert up to 8 characters of the filename. */
     for (i = 0; i < lname && i < 8; i++) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
         char c= toupper(src[i]);
 
         dest[pos++] = valid_d_char(c) ? c : '_';
+
+#else /* Libisofs_old_ecma119_nameS */
+
+        if (dot == NULL && src[i] == '.')
+            dest[pos++] = '_'; /* make sure that ignored dots do not appear */
+        else
+            dest[pos++] = map_fileid_char(src[i], relaxed);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
     }
 
     /* This dot is mandatory, even if there is no extension. */
-    dest[pos++] = '.';
+    if (force_dots || lext > 0)
+        dest[pos++] = '.';
 
     /* Convert up to 3 characters of the extension, if any. */
     for (i = 0; i < lext && i < 3; i++) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
         char c= toupper(src[lname + 1 + i]);
 
         dest[pos++] = valid_d_char(c) ? c : '_';
+
+#else /* Libisofs_old_ecma119_nameS */
+
+        dest[pos++] = map_fileid_char(src[lname + 1 + i], relaxed);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
     }
 
     dest[pos] = '\0';
@@ -817,8 +881,11 @@ char *iso_2_fileid(const char *src)
  * @param size
  *     Max len for the name
  * @param relaxed
- *     0 only allow d-characters, 1 allow also lowe case chars, 
- *     2 allow all characters 
+ *     bit0+1: 0 only allow d-characters,
+ *             1 allow also lowe case chars, 
+ *             2 allow all 8-bit characters,
+ *     bit2:   allow 7-bit characters (but map lowercase to uppercase if
+ *             not bit0+1 == 2)
  */
 char *iso_r_dirid(const char *src, int size, int relaxed)
 {
@@ -833,6 +900,9 @@ char *iso_r_dirid(const char *src, int size, int relaxed)
     if (dest == NULL)
         return NULL;
     for (i = 0; i < len; i++) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
         char c= src[i];
         if (relaxed == 2) {
             /* all chars are allowed */
@@ -853,6 +923,13 @@ char *iso_r_dirid(const char *src, int size, int relaxed)
                 dest[i] = '_';
             }
         }
+
+#else /* Libisofs_old_ecma119_nameS */
+
+        dest[i] = map_fileid_char(src[i], relaxed);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
     }
 
     dest[len] = '\0';
@@ -860,13 +937,17 @@ char *iso_r_dirid(const char *src, int size, int relaxed)
 }
 
 /**
- * Create a file name suitable for an ISO image with relaxed constraints.
+ * Create a file name suitable for an ISO image with level > 1 and
+ * with relaxed constraints.
  * 
  * @param len
  *     Max len for the name, without taken the "." into account.
  * @param relaxed
- *     0 only allow d-characters, 1 allow also lowe case chars, 
- *     2 allow all characters 
+ *     bit0+1: 0 only allow d-characters,
+ *             1 allow also lowe case chars, 
+ *             2 allow all 8-bit characters,
+ *     bit2:   allow 7-bit characters (but map lowercase to uppercase if
+ *             not bit0+1 == 2)
  * @param forcedot
  *     Whether to ensure that "." is added
  */
@@ -913,6 +994,9 @@ char *iso_r_fileid(const char *src, size_t len, int relaxed, int forcedot)
 
     /* Convert up to lnname characters of the filename. */
     for (i = 0; i < lnname; i++) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
         char c= src[i];
         if (relaxed == 2) {
             /* all chars are allowed */
@@ -933,6 +1017,13 @@ char *iso_r_fileid(const char *src, size_t len, int relaxed, int forcedot)
                 dest[pos++] = '_';
             }
         }
+
+#else /* Libisofs_old_ecma119_nameS */
+
+        dest[pos++] = map_fileid_char(src[i], relaxed);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
     }
     if (lnext > 0 || forcedot) {
         dest[pos++] = '.';
@@ -940,6 +1031,9 @@ char *iso_r_fileid(const char *src, size_t len, int relaxed, int forcedot)
 
     /* Convert up to lnext characters of the extension, if any. */
     for (i = lname + 1; i < lname + 1 + lnext; i++) {
+
+#ifdef Libisofs_old_ecma119_nameS
+
         char c= src[i];
         if (relaxed == 2) {
             /* all chars are allowed */
@@ -960,6 +1054,13 @@ char *iso_r_fileid(const char *src, size_t len, int relaxed, int forcedot)
                 dest[pos++] = '_';
             }
         }
+
+#else /* Libisofs_old_ecma119_nameS */
+
+        dest[pos++] = map_fileid_char(src[i], relaxed);
+
+#endif /* ! Libisofs_old_ecma119_nameS */
+
     }
     dest[pos] = '\0';
 
