@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -171,7 +171,7 @@ int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, char *show_adr,
                          int flag)
 {
  int ret, hret, not_writeable= 0, has_what, aquire_flag, load_lba, ext;
- int lba, track, session, params_flag, adr_mode, read_ret;
+ int lba, track, session, params_flag, adr_mode, read_ret, start_lba;
  uint32_t size, offst;
  struct burn_drive_info *dinfo= NULL, *out_dinfo= NULL, *in_dinfo= NULL;
  struct burn_drive *drive= NULL, *out_drive= NULL, *in_drive= NULL;
@@ -384,6 +384,11 @@ int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, char *show_adr,
        goto ex;
    } else {
      iso_image_ref((IsoImage *) xorriso->in_volset_handle);
+     start_lba= -1;
+     ret= Xorriso_get_drive_handles(xorriso, &in_dinfo, &in_drive,
+                        "on attempt to attach ISO image object to outdev", 16);
+     if(ret > 0)
+       start_lba= isoburn_get_attached_start_lba(in_drive);
      ret= isoburn_attach_image(drive, (IsoImage *) xorriso->in_volset_handle);
      if(ret<=0) {
        sprintf(xorriso->info_text,
@@ -391,6 +396,8 @@ int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, char *show_adr,
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FATAL", 0);
        {ret= -1; goto ex;}
      }
+     if(start_lba >= 0)
+       isoburn_attach_start_lba(drive, lba, 0);
    }
    if(!(flag&32))
      Xorriso_toc(xorriso, 1 | 2 | 8);
@@ -422,6 +429,12 @@ int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, char *show_adr,
  if(ret<=0)
    goto ex;
 
+ ret= Xorriso_set_data_cache(xorriso, ropts, xorriso->cache_num_tiles,
+                             xorriso->cache_tile_blocks,
+                             xorriso->cache_default);
+ if(ret<=0)
+   goto ex;
+
  ext= isoburn_ropt_noiso1999;
  if((xorriso->ino_behavior & (1 | 2)) && !(xorriso->do_aaip & (1 | 4 | 32))
     && !(xorriso->do_md5 & 1)) 
@@ -525,12 +538,19 @@ int Xorriso_aquire_drive(struct XorrisO *xorriso, char *adr, char *show_adr,
 
  if(xorriso->out_drive_handle != NULL &&
     xorriso->out_drive_handle != xorriso->in_drive_handle) {
+   start_lba= -1;
+   ret= Xorriso_get_drive_handles(xorriso, &in_dinfo, &in_drive,
+                        "on attempt to attach ISO image volset to outdev", 16);
+   if(ret > 0)
+     start_lba= isoburn_get_attached_start_lba(in_drive);
    ret= Xorriso_get_drive_handles(xorriso, &out_dinfo, &out_drive,
                          "on attempt to attach ISO image volset to outdev", 2);
    if(ret<=0)
      goto ex;
    iso_image_ref((IsoImage *) xorriso->in_volset_handle);
    isoburn_attach_image(out_drive, xorriso->in_volset_handle);
+   if(start_lba >= 0)
+     isoburn_attach_start_lba(out_drive, ret, 0);
  }
  Xorriso_process_msg_queues(xorriso,0);
  isoburn_ropt_get_size_what(ropts, &size, &has_what);
@@ -1384,7 +1404,7 @@ int Xorriso_tell_media_space(struct XorrisO *xorriso,
               isoburn_disc_available_space(drive, burn_options) / (off_t) 2048;
  burn_write_opts_free(burn_options);
 
- if(xorriso->volset_change_pending) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
    ret= Xorriso_write_session(xorriso, 1);
    if(ret>0)
      (*free_space)-= ret;
@@ -2114,9 +2134,12 @@ int Xorriso_check_interval(struct XorrisO *xorriso, struct SpotlisT *spotlist,
  int tag_type= 0, valid, was_sb_tag= 0, in_track_gap= 0;
  char *comparison= "", *sev_text= "DEBUG", *tag_type_name= "";
 
- Xorriso_alloc_meM(data, char, 64*1024);
- if(read_chunk > 32)
-   read_chunk= 32;
+ if(read_chunk > 1024)
+   read_chunk= 1024;
+ else if(read_chunk < 1)
+   read_chunk= 1;
+ Xorriso_alloc_meM(data, char, 2 * read_chunk * 1024);
+
 
  ret= Xorriso_get_drive_handles(xorriso, &dinfo, &drive,
                                 "on attempt to check media readability",
@@ -2124,9 +2147,11 @@ int Xorriso_check_interval(struct XorrisO *xorriso, struct SpotlisT *spotlist,
  if(ret<=0)
    goto ex;
  ret= burn_disc_get_profile(drive, &profile_no, profile_name);
- if(ret > 0)
+ if(ret > 0) {
    if(profile_no >= 0x08 && profile_no <= 0x0a)
      is_cd= 1;
+ }
+ 
 
  if(job->sector_map != NULL) {
    Sectorbitmap_get_layout(job->sector_map, &sectors, &sector_size, 0);
@@ -2422,7 +2447,8 @@ failed_to_write:;
      if(post_read_time - xorriso->last_update_time >=
         xorriso->pacifier_interval)
        Xorriso_pacifier_callback(xorriso, "blocks read",
-                 xorriso->pacifier_count, xorriso->pacifier_total, "", 8 | 16);
+                 xorriso->pacifier_count, xorriso->pacifier_total, "",
+                 8 | 16 | (128 * (job->use_dev == 1)));
    }
  }
  if(prev_quality >= 0) {
@@ -2477,6 +2503,7 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
  char *toc_info= NULL, profile_name[80];
  struct burn_drive *drive;
  struct burn_drive_info *dinfo;
+ enum burn_disc_status s;
  struct isoburn_toc_disc *isoburn_disc= NULL;
  struct isoburn_toc_session **isoburn_sessions;
  struct isoburn_toc_track **iso_burn_tracks;
@@ -2530,6 +2557,14 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
    if(ret <= 0)
      goto ex;
  }
+
+ s= isoburn_disc_get_status(drive);
+ if(s != BURN_DISC_APPENDABLE && s != BURN_DISC_FULL) {
+   Xorriso_msgs_submit(xorriso, 0, "-check_media: No readable medium found",
+                       0, "SORRY", 0);
+   ret= 0; goto ex;
+ }
+
  ret= burn_get_read_capacity(drive, &read_capacity, 0);
  if(ret <= 0)
    read_capacity= -1;
@@ -2617,18 +2652,31 @@ int Xorriso_check_media(struct XorrisO *xorriso, struct SpotlisT **spotlist,
      }
    }
 
- } else if(mode == 1) { /* isoburn disc capacity */
+ } else if(mode == 1) { /* Image range */
+   /* Default is the emulated disc capacity.
+   */
    isoburn_disc= isoburn_toc_drive_get_disc(drive);
    if(isoburn_disc == NULL)
      goto libburn_whole_disc;
    blocks= media_blocks= isoburn_toc_disc_get_sectors(isoburn_disc);
+
+   /* If possible, determine the end address of the loaded ISO image.
+   */
+   track_lba= isoburn_get_attached_start_lba(drive);
+   if(track_lba >= 0) {
+     ret= isoburn_read_iso_head(drive, track_lba, &track_blocks, NULL, 0);
+     if(ret > 0) {
+       blocks= media_blocks= track_lba + track_blocks;
+     }
+   }
+
    if(start_lba >= 0)
      blocks-= start_lba;
    if(media_blocks <= 0)
      goto libburn_whole_disc;
    xorriso->pacifier_total= blocks;
    ret= Xorriso_check_interval(xorriso, *spotlist, job, start_lba, blocks,
-                               read_chunk, 0, 0);
+                               read_chunk, 0, (4 * (xorriso->do_md5 & 1)));
    if(ret <= 0)
      goto ex;
  } else if(mode == 2) {
@@ -2644,14 +2692,14 @@ libburn_whole_disc:;
    blocks-= start_lba;
    xorriso->pacifier_total= blocks;
    ret= Xorriso_check_interval(xorriso, *spotlist, job, start_lba, blocks,
-                               read_chunk, 0, 0);
+                               read_chunk, 0, (4 * (xorriso->do_md5 & 1)));
    if(ret <= 0)
      goto ex;
  }
 
  Xorriso_pacifier_callback(xorriso, "blocks read",
                       xorriso->pacifier_count, xorriso->pacifier_total, "",
-                      1 | 8 | 16 | 32);
+                      1 | 8 | 16 | 32 | (128 * (job->use_dev == 1)));
  ret= 1;
 ex:;
 
