@@ -598,6 +598,7 @@ int Xorriso_genisofs_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "--stdio_sync", "--quoted_path_list", "--efi-boot", "--embedded-boot",
     "-isohybrid-mbr", "-e", "-partition_offset", "-partition_hd_cyl",
     "-partition_sec_hd", "-partition_cyl_align", "-untranslated_name_len",
+    "-rr_reloc_dir",
     ""
  };
  static char arg2_options[][41]= {
@@ -650,10 +651,10 @@ int Xorriso_genisofs_ignore(struct XorrisO *xorriso, char *whom,
  /* mkisofs 2.01 options which are not scheduled for implementation, yet */
  static char ignored_arg0_options[][41]= {
    "-allow-leading-dots", "-ldots", "-allow-multidot",
-   "-cache-inodes", "-no-cache-inodes", "-check-oldnames", "-D",
-   "-disable-deep-relocation",
-   "-L", "-nobak",
-   "-no-bak", "-force-rr", "-relaxed-filenames", "-T",
+   "-cache-inodes", "-check-oldnames",
+   "-L", "-no-bak", "-no-cache-inodes",
+   "-no-split-symlink-components", "-no-split-symlink-fields", "-nobak",
+   "-force-rr", "-T",
    "-no-iso-translate",
    ""
  };
@@ -722,10 +723,15 @@ int Xorriso_genisofs_help(struct XorrisO *xorriso, int flag)
 "  -U, -untranslated-filenames Allow Untranslated filenames (for HPUX & AIX - violates ISO9660).",
 "  -untranslated_name_len LEN  Allow up to LEN (1..96) name characters (heavily violates ISO9660).",
 "  -allow-lowercase            Allow lower case characters in addition to the current character set (violates ISO9660)",
+"  -relaxed-filenames          Allow 7 bit ASCII except lower case characters (violates ISO9660)",
 "  -d, -omit-period            Omit trailing periods from filenames (violates ISO9660)",
 "  -l, -full-iso9660-filenames Allow full 31 character filenames for ISO9660 names",
 "  -max-iso9660-filenames      Allow 37 character filenames for ISO9660 names (violates ISO9660)",
 "  -N, -omit-version-number    Omit version number from ISO9660 filename (violates ISO9660)",
+"  -D, -disable-deep-relocation",
+"                              Disable deep directory relocation (violates ISO9660)",
+"  -hide-rr-moved              Relocate deep directories to /.rr_moved",
+"  -rr_reloc_dir NAME          Set deep directory relocation target in root",
 "  -o FILE, -output FILE       Set output file name",
 "  -m GLOBFILE, -exclude GLOBFILE",
 "                              Exclude file name",
@@ -946,7 +952,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  int do_print_size= 0, fd, idx, iso_level= 1, emul_boot= 2;
  int option_b= 0, was_failure= 0, fret, lower_r= 0, zero= 0;
  int dir_mode= -1, file_mode= -1, count, partition_number, allow_dir_id_ext= 1;
- int root_seen= 0, do_md5_mem;
+ int root_seen= 0, do_md5_mem, option_d= 0;
  mode_t mode_and, mode_or;
  int with_boot_image= 0, with_cat_path= 0, with_emul_toc= 0;
  int old_root_md5= 1, old_root_dev= 0, old_root_ino= 1;
@@ -956,7 +962,7 @@ int Xorriso_genisofs(struct XorrisO *xorriso, char *whom,
  char *ept, *add_pt, *eff_path= NULL, *indev= NULL, msc[80], *cpt;
  char *old_root= NULL, *argpt, *hargv[1];
  char *boot_path, partno_text[8], *iso_rr_pt, *disk_pt, *rpt, *wpt;
- char *rm_merge_args[3];
+ char *rm_merge_args[3], *rr_reloc_dir_pt= NULL;
 
  /* >>> This should not be static. Why is it ? */
  static char *sort_weight_args[4]= {"", "-exec", "sort_weight", ""};
@@ -1300,10 +1306,8 @@ illegal_c:;
    } else if(strcmp(argpt, "-J")==0 || strcmp(argpt, "-joliet")==0) {
      xorriso->do_joliet= 1;
    } else if(strcmp(argpt, "-joliet-long")==0) {
-     ret= Xorriso_option_compliance(xorriso,
-                                    "joliet_long_paths:joliet_long_names", 0);
-     if(ret<=0)
-       goto problem_handler_2;
+     Xorriso_relax_compliance(xorriso,
+                              "joliet_long_paths:joliet_long_names", 0);
    } else if(strcmp(argpt, "-graft-points")==0) {
      xorriso->allow_graft_points= 1;
    } else if(strcmp(argpt, "-path-list")==0 ||
@@ -1488,15 +1492,20 @@ not_enough_args:;
              strcmp(argpt, "-omit-version-number") == 0) {
      Xorriso_relax_compliance(xorriso, "omit_version", 0);
    } else if(strcmp(argpt, "-l") == 0 ||
-             strcmp(argpt, "-full-iso9660-filenames") == 0 ||
-             strcmp(argpt, "-max-iso9660-filenames") == 0) {
+             strcmp(argpt, "-full-iso9660-filenames") == 0) {
+     if(xorriso->iso_level <= 2)
+       Xorriso_relax_compliance(xorriso, "iso_9660_level=2", 0);
+     if(iso_level <= 2)
+       iso_level= 2;
+   } else if(strcmp(argpt, "-max-iso9660-filenames") == 0) {
      Xorriso_relax_compliance(xorriso, "long_names", 0);
    } else if(strcmp(argpt, "-d") == 0 ||
              strcmp(argpt, "-omit-period") == 0) {
      Xorriso_relax_compliance(xorriso, "no_force_dots", 0);
    } else if(strcmp(argpt, "-allow-lowercase") == 0) {
      Xorriso_relax_compliance(xorriso, "lowercase", 0);
-
+   } else if(strcmp(argpt, "-relaxed-filenames") == 0) {
+     Xorriso_relax_compliance(xorriso, "7bit_ascii", 0);
    } else if(strcmp(argpt, "-hide") == 0 ||
              strcmp(argpt, "-hide-list") == 0 ||
              strcmp(argpt, "-hide-joliet") == 0 ||
@@ -1626,6 +1635,31 @@ not_enough_args:;
    } else if(strcmp(argpt, "--no_rc")==0) {
      /* was already handled in Xorriso_prescan_args */;
 
+   } else if(strcmp(argpt, "-D") == 0 ||
+             strcmp(argpt, "-disable-deep-relocation") == 0) {
+     option_d= 1;
+
+   } else if(strcmp(argpt, "-hide-rr-moved") == 0) {
+     rr_reloc_dir_pt= ".rr_moved";
+     goto rr_reloc_dir;
+
+   } else if(strcmp(argpt, "-rr_reloc_dir") == 0) {
+     i++;
+     rr_reloc_dir_pt= argv[i];
+rr_reloc_dir:;
+     if(rr_reloc_dir_pt[0] == '/')
+        rr_reloc_dir_pt++;
+     if(strchr(rr_reloc_dir_pt, '/') != NULL) {
+       sprintf(xorriso->info_text,
+        "-as %s -rr_reloc_dir %s : May only use directories in root directory",
+               whom, Text_shellsafe(argv[i], sfe, 0));
+       Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE",0);
+     }
+     ret= Xorriso_option_rr_reloc_dir(xorriso, rr_reloc_dir_pt, 0);
+     if(ret <= 0)
+       goto problem_handler_2;
+     Xorriso_relax_compliance(xorriso, "deep_paths_off:long_paths_off", 0);
+
    } else if(argpt[0]=='-' && argpt[1]!=0) {
      hargv[0]= argpt;
      ret= Xorriso_genisofs_count_args(xorriso, argc - i, hargv, &count, 1);
@@ -1654,9 +1688,16 @@ is_pathspec_2:;
          pathspec[0]= '/';
          pathspec[1]= 0;
          ret= Sfile_leafname(eff_path, pathspec+1, 0);
-         if(ret>0)
+         if(ret>0) {
+           ret= Fileliste__escape_source_path(pathspec, SfileadrL, 0);
+           if(ret <= 0) {
+             Xorriso_msgs_submit(xorriso, 0,
+                                 "Escaped leaf name gets much too long",
+                                 0, "FAILURE", 0);
+             goto problem_handler_2;
+           }
            strcat(pathspec, "=");
-         else
+         } else
            pathspec[0]= 0;
        }
        strcat(pathspec, eff_path);
@@ -1798,6 +1839,9 @@ problem_handler_2:;
    goto ex;
  }
 
+ if(option_d)
+   Xorriso_relax_compliance(xorriso, "deep_paths:long_paths", 0);
+
  /* After all pathspecs are added: perform boot related options */
  for(j= 0; j < delay_opt_count; j++) {
    i= delay_opt_list[j] & ~(1 << 31);
@@ -1967,7 +2011,7 @@ ex:;
  if(was_path && (!do_print_size) && !old_root[0])
    Xorriso_pacifier_callback(xorriso, "files added", xorriso->pacifier_count,
                              xorriso->pacifier_total, "", 1);
- if(do_print_size && xorriso->volset_change_pending == 1)
+ if(do_print_size && Xorriso_change_is_pending(xorriso, 1))
    xorriso->volset_change_pending= 2;
  if(weight_list != NULL)
    free(weight_list);
