@@ -268,10 +268,10 @@ int Xorriso_set_filter(struct XorrisO *xorriso, void *in_node,
  struct Xorriso_lsT *found_lst;
  struct Xorriso_extF *found_filter;
  IsoExternalFilterCommand *cmd = NULL;
- char *old_name= NULL, new_name[SfileadrL], *suffix= "";
+ char *old_name= NULL, *new_name= NULL, *suffix= "";
  IsoStream *stream;
- int is_renamed= 0;
 
+ Xorriso_alloc_meM(new_name, char, SfileadrL);
  new_name[0]= 0;
 
  node= (IsoNode *) in_node;
@@ -336,7 +336,6 @@ int Xorriso_set_filter(struct XorrisO *xorriso, void *in_node,
                                (flag & 1) | (strip_suffix ? 2 : 0));
    if(ret <= 0 || ret == 2)
      goto ex;
-   is_renamed= 1;
  }
 
  if(strip_filter) {
@@ -376,7 +375,7 @@ int Xorriso_set_filter(struct XorrisO *xorriso, void *in_node,
    filter_ret = iso_file_add_zisofs_filter(file, 1 | (internal_filter & 2));
    if(filter_ret < 0) {
      Xorriso_process_msg_queues(xorriso,0);
-     if(!(internal_filter == 2 && filter_ret == ISO_ZISOFS_WRONG_INPUT))
+     if(!(internal_filter == 2 && filter_ret == (int) ISO_ZISOFS_WRONG_INPUT))
        Xorriso_report_iso_error(xorriso, "", filter_ret,
                     "Error when setting filter to ISO node", 0, "FAILURE", 1);
    }
@@ -431,6 +430,7 @@ int Xorriso_set_filter(struct XorrisO *xorriso, void *in_node,
 ex:;
  if(old_name != NULL)
    free(old_name);
+ Xorriso_free_meM(new_name);
  Xorriso_process_msg_queues(xorriso,0);
  return(ret);
 }
@@ -598,13 +598,13 @@ int Xorriso_status_extf(struct XorrisO *xorriso, char *filter, FILE *fp,
    
    strcpy(xorriso->result_line, "-external_filter ");
    Text_shellsafe(extf->cmd->name, line, 1);
-   if(strlen(line) > maxl)
+   if((int) strlen(line) > maxl)
  continue;
    strcat(line, " ");
    if(extf->cmd->suffix[0]) {
      strcat(line, "suffix=");
      Text_shellsafe(extf->cmd->suffix, line, 1);
-     if(strlen(line) > maxl)
+     if((int) strlen(line) > maxl)
  continue;
      strcat(line, ":");
    }
@@ -617,15 +617,15 @@ int Xorriso_status_extf(struct XorrisO *xorriso, char *filter, FILE *fp,
    if(extf->cmd->behavior & 4)
      strcat(line, "if_block_reduction:");
    sprintf(line + strlen(line), "used=%.f ", (double) extf->cmd->refcount);
-   if(strlen(line) > maxl)
+   if((int) strlen(line) > maxl)
  continue;
    Text_shellsafe(extf->cmd->path, line, 1);
-   if(strlen(line) > maxl)
+   if((int) strlen(line) > maxl)
  continue;
    for(i= 1; i < extf->cmd->argc; i++) {
      strcat(line, " ");
      Text_shellsafe(extf->cmd->argv[i], line, 1);
-     if(strlen(line) > maxl)
+     if((int) strlen(line) > maxl)
    break;
    }
    if(i < extf->cmd->argc)
