@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -95,6 +95,7 @@ int Xorriso_create_empty_iso(struct XorrisO *xorriso, int flag)
  /* Note: no return before isoburn_ropt_destroy() */
  isoburn_ropt_set_extensions(ropts, isoburn_ropt_pretend_blank);
  isoburn_ropt_set_input_charset(ropts, xorriso->in_charset);
+ isoburn_ropt_set_data_cache(ropts, 1, 1, 0);
  isoburn_set_read_pacifier(drive, NULL, NULL);
  ret= isoburn_read_image(drive, ropts, &volset);
  Xorriso_process_msg_queues(xorriso,0);
@@ -872,6 +873,8 @@ int Xorriso_get_volume(struct XorrisO *xorriso, IsoImage **volume,
 }
 
 
+/* @param flag bit0= do not return 1 on volset_change_pending != 1
+*/
 int Xorriso_change_is_pending(struct XorrisO *xorriso, int flag)
 {
  if(flag & 1)
