@@ -863,3 +863,26 @@ int Xorriso_list_extras(struct XorrisO *xorriso, char *mode, int flag)
  return(1);
 }
 
+
+/* @param flag bit0= set num_tiles to default value
+               bit1= set tile_blocks to default value
+*/
+int Xorriso_set_data_cache(struct XorrisO *xorriso, void *o,
+                           int num_tiles, int tile_blocks, int flag)
+{
+ int ret, tiles, blocks, set_flag;
+ struct isoburn_read_opts *ropts;
+
+ ropts= (struct isoburn_read_opts *) o;
+ if(flag & (1 | 2)) {
+   isoburn_ropt_get_data_cache(ropts, &tiles, &blocks, &set_flag, 1);
+   if(flag & 1)
+     num_tiles= tiles;
+   if(flag & 2)
+     tile_blocks= blocks;
+ }
+ ret= isoburn_ropt_set_data_cache(ropts, num_tiles, tile_blocks, 0);
+ return(ret);
+}
+
+
