@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2010 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -1110,6 +1110,28 @@ int Fileliste__target_source_limit(char *line, char sep, char **limit_pt,
 }
 
 
+int Fileliste__escape_source_path(char *line, int size, int flag)
+{
+ int l, count= 0, i;
+ char *wpt;
+
+ l= strlen(line);
+ for(i= 0; i < l; i++)
+   if(line[i] == '=' || line[i] == '\\')
+     count++;
+ if(l + count >= size)
+   return(0);
+
+ wpt= line + l + count;
+ for(i= l; i >= 0; i--) {
+   *(wpt--)= line[i];
+   if(line[i] == '=' || line[i] == '\\')
+     *(wpt--)= '\\';
+ }
+ return(1);
+}
+
+
 int Xorriso__bourne_to_reg(char bourne_expr[], char reg_expr[], int flag)
 /* reg_expr should be twice as large as bourne_expr ( + 2 to be exact) */
 /* return: 2= bourne_expr is surely a constant */
