@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -30,6 +30,28 @@
 #include "xorrisoburn.h"
 
 
+/* Command -data_cache_size */
+int Xorriso_option_data_cache_size(struct XorrisO *xorriso, char *num_tiles,
+                                   char *tile_blocks, int flag)
+{
+ int ret, blocks= -1, tiles= -1, to_default= 0;
+
+ sscanf(num_tiles, "%d", &tiles);
+ sscanf(tile_blocks, "%d", &blocks);
+ if(strcmp(num_tiles, "default") == 0 || num_tiles[0] == 0)
+   to_default|= 1;
+ if(strcmp(tile_blocks, "default") == 0 || tile_blocks[0] == 0)
+   to_default|= 2;
+ ret= Xorriso_set_data_cache(xorriso, NULL, tiles, blocks, to_default);
+ if(ret > 0) {
+   xorriso->cache_num_tiles= tiles;
+   xorriso->cache_tile_blocks= blocks;
+   xorriso->cache_default= to_default;
+ }
+ return(ret);
+}
+
+
 /* Options -dev , -indev, -outdev */
 /** @param flag bit0= use as indev
                 bit1= use as outdev
@@ -60,7 +82,7 @@ int Xorriso_option_dev(struct XorrisO *xorriso, char *in_adr, int flag)
    }
  }
 
- if(xorriso->volset_change_pending && (flag&1)) {
+ if(Xorriso_change_is_pending(xorriso, 0) && (flag&1)) {
    sprintf(xorriso->info_text,
            "%s: Image changes pending. -commit or -rollback first",
            (flag&2) ? "-dev" : "-indev");
@@ -109,7 +131,7 @@ int Xorriso_option_devices(struct XorrisO *xorriso, int flag)
 {
  int ret;
 
- if(xorriso->volset_change_pending) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,
            "-devices: Image changes pending. -commit or -rollback first");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -331,7 +353,7 @@ int Xorriso_option_eject(struct XorrisO *xorriso, char *which, int flag)
    gu_flag|= 2;
  else
    gu_flag|= 3;
- if((gu_flag&1) && xorriso->volset_change_pending) {
+ if((gu_flag&1) && Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,
            "-eject: Image changes pending. -commit or -rollback first");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -358,8 +380,8 @@ int Xorriso_option_end(struct XorrisO *xorriso, int flag)
    cmd= "-rollback_end";
  else
    cmd= "-end";
- if(xorriso->volset_change_pending) {
-   if((flag & 1) || xorriso->volset_change_pending != 1)
+ if(Xorriso_change_is_pending(xorriso, 0)) {
+   if((flag & 1) || !Xorriso_change_is_pending(xorriso, 1))
      which_will= "end the program discarding image changes";
    else
      which_will= "commit image changes and then end the program";
@@ -372,8 +394,8 @@ int Xorriso_option_end(struct XorrisO *xorriso, int flag)
      return(2);
  }
 
- if(xorriso->volset_change_pending) {
-   if((flag & 1) || xorriso->volset_change_pending != 1) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
+   if((flag & 1) || !Xorriso_change_is_pending(xorriso, 1)) {
      xorriso->volset_change_pending= 0;
    } else {
      ret= Xorriso_option_commit(xorriso, 1);
@@ -484,7 +506,7 @@ int Xorriso_option_external_filter(struct XorrisO *xorriso,
  (*idx)= end_idx;
  if(end_idx - start_idx < 3) {
    sprintf(xorriso->info_text,
- "-external_filter : Not enough arguments given. Needed: name options path %s",
+"-external_filter : Not enough parameters given. Needed: name options path %s",
            xorriso->list_delimiter);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
    return(0);
@@ -543,7 +565,7 @@ int Xorriso_option_extract(struct XorrisO *xorriso, char *iso_path,
 
  if(!(flag&2))
    Xorriso_pacifier_callback(xorriso, "files restored",xorriso->pacifier_count,
-                             xorriso->pacifier_total, "", 1|4);
+                             xorriso->pacifier_total, "", 1 | 4 | 8 | 32);
  if(ret <= 0 || problem_count > 0)
    goto ex;
 
@@ -683,7 +705,7 @@ int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
      if(i+1>=end_idx) {
 not_enough_arguments:;
        sprintf(xorriso->info_text,
-               "-find[ix]: not enough arguments with test ");
+               "-find[ix]: not enough parameters with test ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
        goto sorry_ex;
      }
@@ -815,7 +837,7 @@ not_enough_arguments:;
      if(i+1>=end_idx) {
 not_enough_exec_arguments:;
        sprintf(xorriso->info_text,
-               "-find[ix]: not enough arguments with -exec ");
+               "-find[ix]: not enough parameters with -exec ");
        Text_shellsafe(argv[i], xorriso->info_text, 1);
        goto sorry_ex;
      }
@@ -1422,9 +1444,18 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "This program creates, loads, manipulates and writes ISO 9660 filesystem",
 "images with Rock Ridge extensions. Write targets can be drives with optical",
 "media or local filesystem objects.",
+"The program operations are controlled by a sequence of commands, of which",
+"the initial ones are given as program arguments or as lines in startup",
+"files. Further commands may get read from files in batch mode or from",
+"standard input in dialog mode.",
 "",
-"Preparation options:",
-"Drive addresses are either /dev/... as listed with option -devices or",
+"  -x             Only in effect if given as program argument:",
+"                 Execute commands given as program arguments in a sequence",
+"                 that most likely makes some sense. Default is to execute",
+"                 program arguments exactly in the sequence as given.",
+"",
+"Preparation commands:",
+"Drive addresses are either /dev/... as listed with command -devices or",
 "disk files, eventually with prefix \"stdio:\" if non-CD-drive in /dev tree.",
 "E.g. /dev/sr0 , /tmp/pseudo_drive , stdio:/dev/sdc",
 "  -dev address   Set input and output drive and load eventual ISO image.",
@@ -1484,6 +1515,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Allow for writing only the usage of optical drives.",
 "  -early_stdio_test \"on\"|\"appendable_wo\"|\"off\"",
 "                 Classify stdio drives by effective access permissions.",
+"  -data_cache_size number_of_tiles blocks_per_tile",
+"                 Adjust size and granularity of the data read cache.",
 "  -blank \"fast\"|\"all\"|\"deformat\"|\"deformat_quickest\"",
 "                 Blank medium resp. invalidate ISO image on medium.",
 "  -close_damaged \"as_needed\"|\"force\"",
@@ -1520,6 +1553,10 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -compliance rule[:rule...]",
 "                 Allow more or less harmless deviations from strict standards",
 "                 compliance.",
+"  -rr_reloc_dir name",
+"                 Specifies name of relocation directory in root directory,",
+"                 to which deep subtrees will get relocated if -compliance",
+"                 is set to \"deep_paths_off\".",
 "  -boot_image \"any\"|\"isolinux\"|\"grub\"",
 "              |\"discard\"|\"keep\"|\"patch\"|\"dir=\"|\"bin_path=\"",
 "              |\"cat_path=\"|\"cat_hidden=on|iso_rr|joliet|off\"",
@@ -1572,23 +1609,23 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Print foreseeable available space on output medium",
 "  -pvd_info      Print various id strings of the loaded ISO image."
 "",
-"Options with variable length path list [...] need the list delimiter text",
-"as end mark if they are followed by another option. By default this delimiter",
-"is \"--\". In dialog and with options read from files, the line end serves",
-"as such a mark. With program arguments this mark can be omitted only with",
-"the last option in the list of arguments.",
+"Commands with variable length path list [...] need the list delimiter text",
+"as end mark if they are followed by another command. By default this",
+"delimiter is \"--\". In dialog and with commands read from files, the line",
+"end serves as such a mark. With program arguments this mark can be omitted",
+"only with the last command in the list of program arguments.",
 "For brevity the list delimiter is referred as \"--\" throughout this text.",
 "",
 "  -list_delimiter text   Set the list delimiter to be used instead of \"--\"",
 "                 It has to be a single word, must not be empty, not longer",
 "                 than 80 characters, may mot contain quotation marks.",
 "",
-"Manipulation options:",
+"Manipulation commands:",
 "disk_path is a path to an object in the local filesystem tree.",
 "iso_rr_path is the Rock Ridge name of a file object in the ISO image.",
 "pathspec is either a disk_path or (if allowed) a pair: iso_rr_path=disk_path",
-"Options marked by [***] have variable argument length and perform pattern",
-"expansion if enabled by -iso_rr_pattern resp. -disk_pattern.",
+"Commands marked by [***] have variable length parameter lists and perform",
+"pattern expansion if enabled by -iso_rr_pattern resp. -disk_pattern.",
 "",
 "  -pathspecs \"on\"|\"off\"  Allow or disallow pathspecs of form ",
 "                 iso_rr_path=disk_path . Only \"off\" allows eventual",
@@ -1629,8 +1666,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -rm_r iso_rr_path [***]",
 "                 Delete the given directory trees from ISO image.",
 "  -mv iso_rr_path [***] iso_rr_path",
-"                 Rename the given file objects in the ISO tree to the last",
-"                 argument in the list.",
+"                 Rename the given file objects in the ISO tree to iso_rr_path.",
 "  -chown uid iso_rr_path [***]",
 "                 Equivalent to chown in the ISO image.",
 "  -chown_r uid iso_rr_path [***]",
@@ -1685,7 +1721,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                   getfacl, setfacl, getfattr, setfattr, get_any_xattr,",
 "                   list_extattr, get_md5, check_md5, make_md5,",
 "                   set_filter, show_stream, mkisofs_r, hide, find.",
-"                 params are their arguments except iso_rr_path.",
+"                 params are their parameters except iso_rr_path.",
 "  -mkdir iso_rr_path [...]",
 "                 Create empty directories if they do not exist yet.",
 "  -rmdir iso_rr_path [***]",
@@ -1695,8 +1731,8 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -cp_clone iso_rr_path_original [***] iso_rr_path_dest",
 "                 Create ISO to ISO copies according to the rules of cp -r.",
 "",
-"  --             Default list delimiter marking the end of action argument",
-"                 list. It may be changed by option -list_delimiter.",
+"  --             Default list delimiter marking the end of command parameter",
+"                 lists. It may be changed by command -list_delimiter.",
 "",
 "  -not_paths disk_path [***]",
 "                 Add the given paths to the list of excluded absolute paths.",
@@ -1719,7 +1755,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 If \"on\" then ask the user for \"y\" or \"n\" with any",
 "                 file before deleting or overwriting it in the ISO image.",
 "",
-"Filter options:",
+"Filter commands:",
 "External filter processes may produce synthetic file content by reading the",
 "original content from stdin and writing to stdout whatever they want.",
 
@@ -1755,12 +1791,16 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Set global zisofs parameters:",
 "                   level=0|...|9 , block_size=32k|64k|128k , by_magic=on|off",
 "",
-"Write-to-media options:",
+"Write-to-media commands:",
 "  -rollback      Discard the manipulated ISO image and reload it.",
 "",
-"  -commit        Perform the write operation and then perform -dev outdrive.",
+"  -changes_pending \"no\"|\"yes\"|\"mkisofs_printed\"|\"show_status\"",
+"                 Override the automatically determined change status of the",
+"                 loaded image, or show the current status.",
+"  -commit        Perform the write operation if changes are pending.",
+"                 Then perform -dev outdrive.",
 "                 Hint: To perform a final write operation with no new -dev",
-"                       and no new loading of image, execute option -end.",
+"                       and no new loading of image, execute command -end.",
 "  -commit_eject  \"in\"|\"out\"|\"all\"|\"none\"",
 "                 Like -commit but rather eject than load image from outdrive.",
 "                 Give up any unejected drive afterwards.",
@@ -1785,7 +1825,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "                 Immediately eject the medium in -indev, resp. -outdev,",
 "                 resp. both.",
 "",
-"Navigation options:",
+"Navigation commands:",
 "",
 "  -cd iso_rr_path  Change working directory in the ISO image. iso_rr_paths",
 "                 which do not begin with '/' will be inserted beginning at",
@@ -1853,7 +1893,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -show_stream_r iso_rr_path [***]",
 "                 Like -show_stream but affecting all files below directories.",
 "",
-"Restore options which copy file objects from ISO image to disk filesystem:",
+"Restore commands which copy file objects from ISO image to disk filesystem:",
 "  -osirrox \"on\"|\"device_files\"|\"off\"|\"banned\"",
 "           [:\"concat_split_on\"|\"concat_split_off\"]",
 "           [:\"auto_chmod_on\"|\"auto_chmod_off\"]",
@@ -1902,7 +1942,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -check_media_defaults [options] --",
 "                 Preset options for runs of -check_media and -extract_cut.",
 "",
-"Compatibility emulation (argument list may be ended by list delimiter --):",
+"Compatibility emulation (option list may be ended by list delimiter --):",
 "  -as mkisofs  [-help|-version|-o|-R|-r|-J|-V|-P|-f|-m|-exclude-list|",
 "                -no-pad|-M|-C|-graft-points|-path-list|pathspecs|-z|",
 "                -no-emul-boot|-b|-c|-boot-info-table|-boot-load-size|-G|...]",
@@ -1918,7 +1958,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -pacifier \"xorriso\"|\"cdrecord\"|\"mkisofs\"",
 "              Choose format of UPDATE pacifier during write operations.",
 "",
-"General options:",
+"General commands:",
 "  -help       Print this text",
 "  -abort_on severity   Set the threshhold for events to abort the program.",
 "              Useful severities: NEVER, ABORT, FATAL, FAILURE, SORRY, WARNING",
@@ -1933,7 +1973,7 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              Behavior \"best_effort\" is most endurant but may produce",
 "              results which are correct only on the first glimpse.",
 "  -dialog \"on\"|\"off\"|\"single_line\"",
-"              After all arguments are processed, enter dialog mode.",
+"              After all program arguments are processed, enter dialog mode.",
 "              \"single_line\" does not support newline characters within",
 "              open quotation marks and no line continuation by trailing \\.",
 "  -page len width  Prompt user after len output lines (0=no prompt).",
@@ -1962,16 +2002,16 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -logfile channel fileaddress  Copy output of a channel to the given file.",
 "              channel may be 'R','I','M' as with -pkt_output or '.'",
 "              for the consolidated -pkt_output stream.",
-"  -mark text  If text is not empty it will get put out each time an",
-"              option is completed.",
+"  -mark text  If text is not empty it will get put out each time a command",
+"              is completed.",
 "  -temp_mem_limit number[\"k\"|\"m\"]",
 "              Set the maximum size for pattern expansion. (Default is 16m)",
 "  -prog text  Use text as this program's name in subsequent messages",
 "  -prog_help text  Use text as this program's name and perform -help",
-"  -status mode|filter  Report the current settings of persistent options.",
+"  -status mode|filter  Report the current settings of persistent commands.",
 "              Modes:",
-"                 short... print only important or altered options",
-"                 long ... print options even if they have default settings",
+"                 short... print only important or altered settings",
+"                 long ... print settings even if they have default values",
 "                 long_history  like long plus -history: lines",
 "              Filters begin with '-' and are compared literally against the",
 "              output lines of -status long_history. A line is put out only",
@@ -1979,10 +2019,11 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "  -status_history_max number  Maximum number of history lines to be reported",
 "              with -status:long_history",
 "  -options_from_file fileaddress",
-"              Reads lines from the given file and executes them as program",
-"              options.",
-"  -no_rc      Only if used as first command line argument this option",
-"              prevents reading and interpretation of startup files.",
+"              Reads lines from the given file and executes them as commands.",
+"  -no_rc      Only if used as first program argument, this command",
+"              prevents reading and interpretation of these startup files:",
+"               /etc/default/xorriso , /etc/opt/xorriso/rc",
+"               /etc/xorriso/xorriso.conf , $HOME/.xorrisorc",
 "  -print text",
 "              Print a text to result channel.",
 "  -print_info text",
@@ -2006,19 +2047,20 @@ int Xorriso_option_help(struct XorrisO *xorriso, int flag)
 "              Tell whether certain extra features were enabled at compile",
 "              time. Code \"all\" lists all features and a headline. Other",
 "              codes pick a single feature. \"codes\" lists the known codes.",
+"  -list_arg_sorting",
+"              Print the sorting order of xorriso commands with option -x.",
 "  -version    Tell program and version number",
 "  -end        End program. Commit eventual pending changes.",
 "  -rollback_end",
 "              End program. Discard pending changes.",
 "",
 "",
-"Option -page causes a user prompt after the given number of result lines.",
+"Command -page causes a user prompt after the given number of result lines.",
 "Empty input resumes output until the next prompt. Other input may be:",
 "  @     suppresses paging until the current action is done",
 "  @@    suppresses further result output but continues the action",
 "  @@@   aborts the current action",
-"  other aborts the current action and executes input as new",
-"        option",
+"  other aborts the current action and executes input as new command",
 "",
 
 #endif /* ! Xorriso_no_helP */
