@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -74,6 +74,16 @@ int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag)
 }
 
 
+/* Option -list_arg_sorting */
+int Xorriso_option_list_arg_sorting(struct XorrisO *xorriso, int flag)
+{
+ int ret;
+
+ ret= Xorriso_cmd_sorting_rank(xorriso, 0, NULL, 0, 1);
+ return(ret);
+}
+
+
 /* Option -list_delimiter */
 int Xorriso_option_list_delimiter(struct XorrisO *xorriso, char *text,
                                   int flag)
@@ -181,7 +191,7 @@ int Xorriso_option_load(struct XorrisO *xorriso, char *adr_mode,
 {
  int ret;
 
- if(xorriso->volset_change_pending) {
+ if(Xorriso_change_is_pending(xorriso, 0)) {
    sprintf(xorriso->info_text,
            "-load: Image changes pending. -commit or -rollback first");
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
@@ -741,7 +751,7 @@ int Xorriso_option_mount(struct XorrisO *xorriso, char *dev, char *adr_mode,
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
      return(0);
    }
-   if(xorriso->volset_change_pending) {
+   if(Xorriso_change_is_pending(xorriso, 0)) {
      sprintf(xorriso->info_text,
              "%s: Image changes pending. -commit or -rollback first", mnt);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "FAILURE", 0);
