@@ -1,7 +1,7 @@
 
 /* xorriso - creates, loads, manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -450,7 +450,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "ban_stdio_write","close_filter_list","commit",
     "device_links","devices","end",
     "for_backup", "help",
-    "list_formats","list_speeds",
+    "list_arg_sorting","list_formats","list_speeds",
     "no_rc","print_size","pvd_info","pwd","pwdi","pwdx",
     "read_mkisofsrc","rollback","rollback_end",
     "tell_media_space","toc","version",
@@ -460,7 +460,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "abort_on","acl","add_plainly","application_id","auto_charset",
     "abstract_file",
     "backslash_codes","blank","biblio_file",
-    "calm_drive","cd","cdi","cdx","charset","close","close_damaged",
+    "calm_drive","cd","cdi","cdx","changes_pending","charset",
+    "close","close_damaged",
     "commit_eject","compliance","copyright_file",
     "dev","dialog","disk_dev_ino","disk_pattern","displacement",
     "dummy","dvd_obs","early_stdio_test", "eject",
@@ -472,8 +473,8 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
     "pacifier","padding","path_list","pathspecs","pkt_output",
     "preparer_id","print","print_info","print_mark","prompt",
     "prog","prog_help","publisher","quoted_not_list","quoted_path_list",
-    "reassure","report_about","rom_toc_scan","scsi_log",
-    "session_log","signal_handling",
+    "reassure","report_about","rom_toc_scan","rr_reloc_dir","scsi_log",
+    "session_log","signal_handling","sleep",
     "speed","split_size","status","status_history_max",
     "stdio_sync","stream_recording","system_id","temp_mem_limit",
     "uid","unregister_filter","use_readline","volid","volset_id",
@@ -482,6 +483,7 @@ int Xorriso_count_args(struct XorrisO *xorriso, int argc, char **argv,
  };
  static char arg2_commands[][40]= {
     "assert_volid","boot_image","clone","compare","compare_r","drive_class",
+    "data_cache_size",
     "errfile_log","error_behavior","extract","extract_single",
     "jigdo","load","logfile",
     "map","map_single","page","return_with",
@@ -571,9 +573,278 @@ ex:
 }
 
 
+/* @param flag bit0= list sorting order rather than looking for argv[idx]
+*/
+int Xorriso_cmd_sorting_rank(struct XorrisO *xorriso,
+                        int argc, char **argv, int idx, int flag)
+{
+ int ret, i, cmd_data_size= 2 * SfileadrL;
+ char *cmd, *cmd_data= NULL;
+ static char *commands[]= {
+
+   "* Execution order of program arguments with option -x:",
+   "x",
+
+   "* Support for frontend programs via stdin and stdout (1):",
+   "prog", "prog_help",
+
+   "* Exception processing:",
+   "abort_on", "return_with", "report_about", "signal_handling",
+   "error_behavior",
+
+   "* Scripting, dialog and program control features (1):",
+   "no_rc", "help", "version", "list_extras", "list_arg_sorting",
+   "temp_mem_limit", "backslash_codes",
+   "errfile_log", "session_log", "scsi_log",
+   "options_from_file", "list_delimiter",
+   "print", "print_info", "print_mark", "prompt", "sleep",
+
+   "* Drive and media related inquiry actions (1):",
+   "devices", "device_links",
+   "mount_opts", "mount_cmd", "session_string",
+
+   "* Influencing the behavior of image loading:",
+   "load", "displacement", "drive_class", "assert_volid", "in_charset",
+   "auto_charset", "hardlinks", "acl", "xattr", "md5", "for_backup",
+   "disk_dev_ino", "rom_toc_scan", "calm_drive", "ban_stdio_write",
+   "early_stdio_test", "data_cache_size",
+
+   "* Character sets:",
+   "charset", "local_charset",
+
+   "* Aquiring source and target drive:",
+   "dev", "indev", "outdev",
+
+   "* Drive and media related inquiry actions (2):",
+   "list_profiles", "list_formats", "list_speeds",
+   "toc", "pvd_info",
+
+   "* Settings for file insertion:",
+   "file_size_limit", "not_mgt", "not_paths", "not_leaf", "not_list",
+   "quoted_not_list", "follow", "pathspecs", "overwrite", "split_size",
+
+   "* Navigation in ISO image and disk filesystem (1):",
+   "cd", "cdx", "pwd", "pwdx",
+
+   "* Inserting files into ISO image:",
+   "disk_pattern", "add_plainly",
+   "mkdir", "add", "path_list", "quoted_path_list",
+   "map", "map_single", "map_l", "update", "update_r", "update_l",
+   "cut_out", "cpr", 
+   "clone", "cp_clone",
+
+   "* Navigation in ISO image and disk filesystem (2):",
+   "ls", "lsd", "lsl", "lsdl", "lsx", "lsdx", "lslx", "lsdlx",
+   "getfacl", "getfacl_r", "getfattr", "getfattr_r", "du", "dus",
+   "dux", "dusx", "findx",
+   "compare", "compare_r", "compare_l", "show_stream", "show_stream_r",
+
+   "* File manipulations:",
+   "iso_rr_pattern",
+   "rm", "rm_r", "rmdir", "mv",
+   "chown", "chown_r", "chgrp", "chgrp_r", "chmod", "chmod_r", "setfacl",
+   "setfacl_r", "setfacl_list", "setfattr", "setfattr_r", "setfattr_list",
+   "alter_date", "alter_date_r", "hide",
+
+   "* Filters for data file content:",
+   "external_filter", "unregister_filter", "close_filter_list",
+   "set_filter", "set_filter_r", 
+
+   "* Tree traversal command -find:",
+   "find",
+
+   "* osirrox ISO-to-disk restore options:",
+   "osirrox", "extract", "extract_single", "extract_l", "extract_cut",
+   "cpx", "cpax", "cp_rx", "cp_rax", "paste_in",
+   "mount",
+
+   "* Settings for result writing:",
+   "joliet", "compliance", "rr_reloc_dir", "volid", "volset_id", "publisher",
+   "application_id", "system_id", "volume_date", "copyright_file",
+   "abstract_file", "biblio_file", "preparer_id", "out_charset",
+   "read_mkisofsrc",
+   "uid", "gid", "zisofs", "speed", "stream_recording", "dvd_obs",
+   "stdio_sync", "dummy", "fs", "close", "padding",
+   "grow_blindly", "pacifier", "scdbackup_tag",
+
+   "* Bootable ISO images:",
+   "boot_image", "append_partition",
+
+   "* Jigdo Template Extraction:",
+   "jigdo",
+
+   "* Command compatibility emulations:",
+   "as",
+
+   "* Scripting, dialog and program control features (2):",
+   "history", "status_history_max", "status",
+
+   "* Drive and media related inquiry actions (3):",
+   "print_size", "tell_media_space",
+
+   "* Writing the result, drive control:",
+   "format", "blank", "close_damaged",
+   "rollback", "changes_pending", "commit", "commit_eject",
+   "eject",
+
+   "* Evaluation of readability and recovery:",
+   "check_media_defaults", "check_media", "check_md5", "check_md5_r",
+
+   "* Support for frontend programs via stdin and stdout (2):",
+   "pkt_output", "logfile", "mark",
+
+   "* Dialog mode control:",
+   "dialog", "page", "use_readline", "reassure",
+
+   "* Scripting, dialog and program control features (3):",
+   "rollback_end", "end",
+
+   ""
+ };
+
+ if(flag & 1) {
+   for(i= 0; commands[i][0] !=0; i++) {
+     if(commands[i][0] == '*')
+       sprintf(xorriso->result_line, "#%s\n", commands[i] + 1);
+     else
+       sprintf(xorriso->result_line, "-%s\n", commands[i]);
+     Xorriso_result(xorriso, 0);
+   }
+   ret= 1; goto ex;
+ }
+ if(argc <= 0)
+   {ret= -1; goto ex;}
+
+ Xorriso_alloc_meM(cmd_data, char, cmd_data_size);
+ ret= Xorriso_normalize_command(xorriso, argv[idx], -1,
+                                cmd_data, cmd_data_size, &cmd, 0);
+ if(ret < 0)
+   goto ex;
+
+ if(cmd[0] == '#' || cmd[0] == 0 ||
+    strcmp(cmd, xorriso->list_delimiter) == 0) {
+   /* Move to end: comment line , empty option , orphaned list delimiter */
+   ret= 0x7fffffff; goto ex;
+ }
+ for(i= 0; commands[i][0] !=0; i++) {
+   if(commands[i][0] == '*') /* headline in command list */
+ continue;
+   if(strcmp(commands[i], cmd) != 0)
+ continue;
+   ret= i + 1; goto ex;
+ }
+
+ ret= 1;
+ex:
+ Xorriso_free_meM(cmd_data);
+ return(ret);
+}
+
+
+int Xorriso__cmp_cmd_rank(const void *a, const void *b)
+{
+ int ra, rb;
+
+ ra= ((int *) a)[1];
+ rb= ((int *) b)[1];
+ if(ra < rb)
+   return(-1);
+ if(ra > rb)
+   return(1);
+ ra= ((int *) a)[2];
+ rb= ((int *) b)[2];
+ if(ra < rb)
+   return(-1);
+ if(ra > rb)
+   return(1);
+ return(0);
+}
+
+
+/* @param flag bit0= print command sequence rather than executing it
+               bit1= these are command line arguments
+                     (for xorriso->argument_emulation)
+*/
+int Xorriso_exec_args_sorted(struct XorrisO *xorriso,
+                             int argc, char **argv, int *idx, int flag)
+{
+ int cmd_count= 0, ret, i, arg_count, *idx_rank= NULL, cmd_idx;
+
+ /* Count commands and allocate index-rank array */
+ for(i= *idx; i < argc; i++) {
+   ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
+   if(ret <= 0)
+     goto ex;
+   if(ret != 1)
+ continue;
+   cmd_count++;
+   i+= arg_count;
+ }
+ if(cmd_count <= 0)
+   {ret= 1; goto ex;}
+ Xorriso_alloc_meM(idx_rank, int, 3 * cmd_count);
+
+ /* Fill index-rank array and sort */
+ cmd_count= 0;
+ for(i= *idx; i < argc; i++) {
+   ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
+   if(ret <= 0)
+     goto ex;
+   if(ret != 1)
+ continue;
+   idx_rank[3 * cmd_count]= i;
+   ret= Xorriso_cmd_sorting_rank(xorriso, argc, argv, i, 0);
+   if(ret < 0)
+     goto ex;
+   idx_rank[3 * cmd_count + 1]= ret;
+   idx_rank[3 * cmd_count + 2]= cmd_count;
+   cmd_count++;
+   i+= arg_count;
+ }
+ qsort(idx_rank, cmd_count, 3 * sizeof(int), Xorriso__cmp_cmd_rank);
+
+ /* Execute or print indice from index-rank array */
+ if(flag & 1) {
+   sprintf(xorriso->result_line,
+           "Automatically determined command sequence:\n");
+   Xorriso_result(xorriso, 0);
+   xorriso->result_line[0]= 0;
+ }
+ for(i= 0; i < cmd_count; i++) {
+   cmd_idx= idx_rank[3 * i];
+   if(flag & 1) {
+     if(strlen(xorriso->result_line) + 1 + strlen(argv[cmd_idx]) > 78) {
+       strcat(xorriso->result_line, "\n");
+       Xorriso_result(xorriso, 0);
+       xorriso->result_line[0]= 0;
+     }
+     sprintf(xorriso->result_line + strlen(xorriso->result_line),
+             " %s", argv[cmd_idx]);
+   } else {
+     ret= Xorriso_interpreter(xorriso, argc, argv, &cmd_idx, 4 | (flag & 2));
+     if(ret <= 0 || ret == 3)
+       goto ex;
+   }
+ }
+ if(flag & 1) {
+   if(strlen(xorriso->result_line) > 0) {
+     strcat(xorriso->result_line, "\n");
+     Xorriso_result(xorriso, 0);
+   }
+ } else
+   *idx= argc;
+ ret= 1;
+ex:
+ Xorriso_free_meM(idx_rank); 
+ return(ret);
+}
+
+
 /* @param flag bit0= recursion
                bit1= these are command line arguments
                      (for xorriso->argument_emulation)
+               bit2= Only execute the one command argv[*idx] and advance 
+                     *idx to the next command if sucessful. Then return.
 */
 int Xorriso_interpreter(struct XorrisO *xorriso,
                         int argc, char **argv, int *idx, int flag)
@@ -610,15 +881,19 @@ next_command:;
      if((*idx)>=argc)
        {ret= 1; goto ex;}
    }
+   if((xorriso->arrange_args || (flag & 8)) && !(flag & (4 | 16))) {
+     ret= Xorriso_exec_args_sorted(xorriso, argc, argv, idx, 0);
+     goto ex;
+   } 
  }
 
  ret= Xorriso_count_args(xorriso, argc - *idx, argv + *idx, &arg_count, 1);
  if((ret == 1 || ret == 2) &&
     strcmp(argv[*idx], xorriso->list_delimiter) != 0) {
-   sprintf(xorriso->info_text, "Command:  %s", argv[*idx]);
+   sprintf(xorriso->info_text, "Command:    %s", argv[*idx]);
    Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
    for(i= 1; i <= arg_count && *idx + i < argc; i++) {
-     sprintf(xorriso->info_text, "Argument:    %s", argv[*idx + i]);
+     sprintf(xorriso->info_text, "Parameter:     %s", argv[*idx + i]);
      Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
    }
    if(*idx + arg_count >= argc) {
@@ -735,6 +1010,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_cdx(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd, "changes_pending")==0) {
+   (*idx)++;
+   ret= Xorriso_option_changes_pending(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"charset")==0) {
    (*idx)++;
    ret= Xorriso_option_charset(xorriso, arg1, 3);
@@ -841,6 +1120,10 @@ next_command:;
      ret= Xorriso_option_cut_out(xorriso, arg1, arg2,
                                  argv[(*idx)-2], argv[(*idx)-1], 0);
 
+ } else if(strcmp(cmd,"data_cache_size")==0) {
+   (*idx)+= 2;
+   ret= Xorriso_option_data_cache_size(xorriso, arg1, arg2, 0);
+
  } else if(strcmp(cmd,"dev")==0) {
    (*idx)++;
    ret= Xorriso_option_dev(xorriso, arg1, 3);
@@ -1019,6 +1302,9 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_joliet(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd, "list_arg_sorting") == 0) {
+   ret= Xorriso_option_list_arg_sorting(xorriso, 0);
+
  } else if(strcmp(cmd, "list_delimiter") == 0) {
    (*idx)++;
    ret= Xorriso_option_list_delimiter(xorriso, arg1, 0);
@@ -1278,6 +1564,10 @@ next_command:;
    (*idx)++;
    Xorriso_option_rom_toc_scan(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"rr_reloc_dir")==0) {
+   (*idx)++;
+   Xorriso_option_rr_reloc_dir(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"scdbackup_tag")==0) {
    (*idx)+= 2;
    ret= Xorriso_option_scdbackup_tag(xorriso, arg1, arg2, 0);
@@ -1339,6 +1629,10 @@ next_command:;
    (*idx)++;
    ret= Xorriso_option_signal_handling(xorriso, arg1, 0);
 
+ } else if(strcmp(cmd,"sleep")==0) {
+   (*idx)++;
+   ret= Xorriso_option_sleep(xorriso, arg1, 0);
+
  } else if(strcmp(cmd,"speed")==0) {
    (*idx)++;
    ret= Xorriso_option_speed(xorriso, arg1, 0);
@@ -1419,6 +1713,9 @@ next_command:;
    (*idx)+= 2;
    ret= Xorriso_option_volume_date(xorriso, arg1, arg2, 0);
 
+ } else if(strcmp(cmd, "x") == 0) {
+   /* only in effect in Xorriso_prescan_args() */;
+
  } else if(strcmp(cmd,"xattr")==0) {
    (*idx)++;
    ret= Xorriso_option_xattr(xorriso, arg1, 0);
@@ -1455,7 +1752,7 @@ eval_any_problems:
  if(ret<0)
    goto ex;
 
- if(*idx<argc)
+ if(*idx < argc && !(flag & 4))
    goto next_command;
 
 ex:;
@@ -1585,6 +1882,7 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
                          int flag)
 /*
  bit0= do not interpret argv[1]
+ bit1= complain about inknown arguments
 */
 /*
  return:
@@ -1643,9 +1941,19 @@ int Xorriso_prescan_args(struct XorrisO *xorriso, int argc, char **argv,
      ret= Xorriso_option_no_rc(xorriso, 0);
      if(ret<=0)
        error_seen= 1;
-   } else if(xorriso->argument_emulation >= 1 &&
-             xorriso->argument_emulation <= 2) {
      {ret= 1; goto ex;}
+   } else if(xorriso->argument_emulation == 1) { /* mkisofs emulation */
+     if(xorriso->dev_fd_1 < 0)
+       goto protect_stdout;
+     {ret= 1; goto ex;}
+
+   } else if(xorriso->argument_emulation == 2) { /* cdrecord emulation */
+     if(xorriso->dev_fd_1 < 0)
+       if(Xorriso_cdrskin_uses_stdout(xorriso, argc - 1 - (flag & 1),
+                                      argv + 1 + (flag & 1), 0))
+         goto protect_stdout;
+     {ret= 1; goto ex;}
+
    } else if((strcmp(cmd,"dev")==0 || strcmp(cmd,"outdev")==0 ||
                                       strcmp(cmd,"indev")==0) &&
              (strcmp(arg1,"stdio:/dev/fd/1")==0 || strcmp(arg1,"-")==0) &&
@@ -1666,6 +1974,8 @@ protect_stdout:;
        sprintf(xorriso->info_text, "Disabling use of libreadline.");
        Xorriso_msgs_submit(xorriso, 0, xorriso->info_text, 0, "DEBUG", 0);
      }
+     if(xorriso->argument_emulation >= 1 && xorriso->argument_emulation <=2)
+       {ret= 1; goto ex;}
 
    } else if(strcmp(cmd,"abort_on")==0 && was_dashed == 1) {
      i++;
@@ -1689,12 +1999,22 @@ protect_stdout:;
 
    } else if(strcmp(cmd,"as")==0 && was_dashed == 1) {
      ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
-     if(ret == 1)
+     if(ret == 1) {
        i+= arg_count;
-     if(was_dashed == 1)
-       if(strcmp(arg1, "mkisofs")==0 || strcmp(arg1, "genisoimage")==0 ||
-          strcmp(arg1, "genisofs")==0 || strcmp(arg1, "xorrisofs")==0)
+
+       if((strcmp(arg1, "cdrecord")==0 || strcmp(arg1, "wodim")==0 ||
+           strcmp(arg1, "cdrskin")==0 || strcmp(arg1, "xorrecord")==0) &&
+          xorriso->dev_fd_1 < 0)
+         if(Xorriso_cdrskin_uses_stdout(xorriso, arg_count - 1,
+                                        argv + i - arg_count + 2, 0))
+           goto protect_stdout;
+     }
+     if(was_dashed == 1) {
+       if((strcmp(arg1, "mkisofs")==0 || strcmp(arg1, "genisoimage")==0 ||
+           strcmp(arg1, "genisofs")==0 || strcmp(arg1, "xorrisofs")==0) &&
+          xorriso->dev_fd_1 < 0)
          goto protect_stdout;
+     }
 
    } else if(strcmp(cmd, "list_delimiter") == 0) {
      /* Needed for interpreting other args. Gets reset after prescan. */
@@ -1724,6 +2044,9 @@ protect_stdout:;
        Xorriso_option_signal_handling(xorriso, arg1, 1); /* no install */
      was_signal_handling= 1;
 
+   } else if(strcmp(original_cmd, "-x") == 0) {
+     xorriso->arrange_args= 1;
+
    } else {
      ret= Xorriso_count_args(xorriso, argc - i, argv + i, &arg_count, 1);
      if(ret == 1) {
@@ -2362,7 +2685,7 @@ int Xorriso_normalize_acl_text(struct XorrisO *xorriso, char *in_text,
    if(pass == 0) {
      *access_acl_text= calloc(access_count + 1, 1);
      *default_acl_text= calloc(default_count + 1, 1);
-     if(access_acl_text == NULL || *default_acl_text == NULL) {
+     if(*access_acl_text == NULL || *default_acl_text == NULL) {
        Xorriso_no_malloc_memory(xorriso, access_acl_text, 0);
        {ret= -1; goto ex;}
      }
