@@ -15,7 +15,7 @@
    dialog tool xorriso.
 
    Alternatively to command interpreters it is possible to run all options of
-   xorriso directly via the calls of the "Options API". 
+   xorriso directly via the calls of the "Command API". 
    The "Problem Status and Message API" shall then be used to obtain the
    text output of the options.
 
@@ -61,7 +61,7 @@ struct XorrisO;
 */
 #define Xorriso_header_version_majoR  1
 #define Xorriso_header_version_minoR  2
-#define Xorriso_header_version_micrO  0
+#define Xorriso_header_version_micrO  2
 
 
 /** If needed: Something like ".pl01" to indicate a bug fix. Normally empty.
@@ -172,7 +172,7 @@ int Xorriso_new(struct XorrisO ** xorriso, char *progname, int flag);
          Xorriso_prescan_args().
          The other command interpreters may be used only after
          Xorriso_startup_libraries(). The same restriction applies to the
-         calls of the Options API further below.
+         calls of the Command API further below.
 */
 
 
@@ -191,7 +191,7 @@ int Xorriso_startup_libraries(struct XorrisO *xorriso, int flag);
 
 
 /* Note: After library startup, you may run Command Interpreters or call
-         functions from the Options API.
+         functions from the Command API.
 
          Wenn all desired activities are done, you may check whether there are
          uncommited changes pending, compute an exit value, destroy the XorrisO
@@ -236,6 +236,8 @@ int Xorriso_destroy(struct XorrisO **xorriso, int flag);
    get into effect before the libraries get initialized:
      -abort_on , -report_about , -return_with , -list_delimiter ,
      -scsi_log , -signal_handling
+   This is the only occasion where command -x has an effect:
+     -x
    Some commands get executed only if they are the only command in argv:
      -prog_help , -help
    The following is recognized only if it is the first of all arguments:
@@ -298,9 +300,15 @@ int Xorriso_program_arg_bsl(struct XorrisO *xorriso, int argc, char ***argv,
 
 /* Interpret argv as xorriso command options and their parameters.
    (An alternative is to call functions of the options API directly and to
-    perform own error status evaluation. See below: Options API.) 
+    perform own error status evaluation. See below: Command API.) 
    After the first command and its parameters there may be more commands and
    parameters. All parameters must be given in the same call as their command.
+   @since 1.2.2:
+   Commands may get arranged in a sequence that is most likely to make sense.
+   E.g. image loading settings before drive aquiration, then commands fori
+   adding files, then settings for writing, then writing.
+   This feature may be enabled by command "-x" in Xorriso_prescan_args()
+   or by parameter flag of this call.
    @param xorriso The context object in which to perform the commands.
    @param argc    Number of arguments.
    @param argv    The arguments. argv[0] contains the program name.
@@ -308,10 +316,24 @@ int Xorriso_program_arg_bsl(struct XorrisO *xorriso, int argc, char ***argv,
    @param idx     Argument cursor. When this function is called, *idx must
                   be at least 1, argv[*idx] must be a command.
                   *idx will iterate over commands and parameters until this
-                  function aborts or until argc is reached.
+                  function aborts, or until argc is reached, or only once if
+                  flag bit2 is set.
    @param flag    bit0= reserved. Indicates recursion. Submit 0. 
                   bit1= Indicates that these are the main() program start
-                        arguments
+                        arguments. This enables their use with emulations
+                        which where set with Xorriso_new(), or argument
+                        arranging.
+                  bit2= Only execute the one command argv[*idx] and advance
+                        *idx to the next command if sucessful. Then return.
+                        This prevents any argument arranging.
+                        @since 1.2.2
+                  bit3= With bit1 and not bit2:
+                        Enable argument arranging as with
+                        Xorriso_prescan_args() and command "-x".
+                        @since 1.2.2
+                  bit4= With bit1:
+                        Surely disable argument arranging.
+                        @since 1.2.2
    @return <=0 = error
              1 = success
              2 = problem event ignored
@@ -567,21 +589,21 @@ int Xorriso_lst_destroy_all(struct Xorriso_lsT **lstring, int flag);
 
 
 
-/* ---------------------------- Options API ------------------------ */
-/*    See man 1 xorriso for explanation of the particular options    */
+/* ---------------------------- Command API ------------------------ */
+/*    See man 1 xorriso for explanation of the particular commands   */
 /*
-   Before each call to an option function, there should happen:
+   Before each call to a command function, there should happen:
    Xorriso_set_problem_status() with empty severity text.
 
-   After each call to an option function, there should happen:
+   After each call to a command function, there should happen:
    Xorriso_eval_problem_status()
    One should follow its eventual advice to abort.
 
-   Options with a varying number of arguments get then passed like
+   Commands with a varying number of arguments get then passed like
    Xorriso_interpreter(). E.g.:
      int Xorriso_option_add(struct XorrisO *xorriso, int argc, char **argv,
                             int *idx, int flag);
-   The option functions will begin to read the arguments at position *idx
+   The command functions will begin to read the arguments at position *idx
    and will see the list end either at the next argv which contains the
    -list_delimiter text or at argv[argc-1].
    After the call, *idx will be the index of the first not yet interpreted
@@ -595,96 +617,101 @@ int Xorriso_lst_destroy_all(struct Xorriso_lsT **lstring, int flag);
 */
 
 
-/* Option -abort_on */
+/* Command -abort_on */
 int Xorriso_option_abort_on(struct XorrisO *xorriso, char *severity, int flag);
 
-/* Option -abstract_file */
+/* Command -abstract_file */
 /* @since 0.6.0 */
 int Xorriso_option_abstract_file(struct XorrisO *xorriso, char *name,
                                  int flag);
 
-/* Option -acl "on"|"off" */
+/* Command -acl "on"|"off" */
 int Xorriso_option_acl(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -add */
+/* Command -add */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
 */
 int Xorriso_option_add(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag);
 
-/* Option -add_plainly "on"|"off" */
+/* Command -add_plainly "on"|"off" */
 int Xorriso_option_add_plainly(struct XorrisO *xorriso, char *mode,
                                    int flag);
 
 
-/* Option -alter_date, alter_date_r */
+/* Command -alter_date, alter_date_r */
 /* @param flag bit0=recursive (-alter_date_r)
 */
 int Xorriso_option_alter_date(struct XorrisO *xorriso, 
                               char *time_type, char *timestring,
                               int argc, char **argv, int *idx, int flag);
 
-/* Option -append_partition */
+/* Command -append_partition */
 /* @since 0.6.4 */
 int Xorriso_option_append_partition(struct XorrisO *xorriso, char *partno_text,
                                   char *type_text, char *image_path, int flag);
 
-/* Option -application_id */
+/* Command -application_id */
 int Xorriso_option_application_id(struct XorrisO *xorriso, char *name, 
                                   int flag);
 
-/* Option -as */
+/* Command -as */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
 */  
 int Xorriso_option_as(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
 
-/* Option -assert_volid */
+/* Command -assert_volid */
 int Xorriso_option_assert_volid(struct XorrisO *xorriso, char *pattern,
                                 char *severity, int flag);
 
-/* Option -auto_charset "on"|"off" */
+/* Command -auto_charset "on"|"off" */
 int Xorriso_option_auto_charset(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -backslash_codes */
+/* Command -backslash_codes */
 int Xorriso_option_backslash_codes(struct XorrisO *xorriso, char *mode,
                                    int flag);
 
-/* Option -ban_stdio_write */
+/* Command -ban_stdio_write */
 int Xorriso_option_ban_stdio_write(struct XorrisO *xorriso, int flag);
 
-/* Option -biblio_file */
+/* Command -biblio_file */
 /* @since 0.6.0 */
 int Xorriso_option_biblio_file(struct XorrisO *xorriso, char *name, int flag);
 
-/* Option -blank and -format */
+/* Command -blank and -format */
 /* @param flag bit0= format rather than blank
    @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_blank(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -boot_image */
+/* Command -boot_image */
 int Xorriso_option_boot_image(struct XorrisO *xorriso, char *form,
                               char *treatment, int flag);
 
-/* Option -calm_drive */
+/* Command -calm_drive */
 int Xorriso_option_calm_drive(struct XorrisO *xorriso, char *which, int flag);
 
-/* Option -cd alias -cdi */
+/* Command -cd alias -cdi */
 int Xorriso_option_cdi(struct XorrisO *xorriso, char *iso_rr_path, int flag);
 
-/* Option -cdx */
+/* Command -cdx */
 int Xorriso_option_cdx(struct XorrisO *xorriso, char *disk_path, int flag);
 
-/* Option -charset */
+/* Command -changes_pending */
+/* @since 1.2.2 */
+int Xorriso_option_changes_pending(struct XorrisO *xorriso, char *state,
+                                   int flag);
+
+/* Command -charset */
 /* @param flag bit0= set in_charset
                bit1= set out_charset
 */
 int Xorriso_option_charset(struct XorrisO *xorriso, char *name, int flag);
 
-/* Options -check_md5 and -check_md5_r
+/* Command -check_md5 and -check_md5_r
    @param flag bit0= issue summary message
                bit1= do not reset pacifier, no final pacifier message
                bit2= do not issue pacifier messages at all
@@ -693,62 +720,62 @@ int Xorriso_option_charset(struct XorrisO *xorriso, char *name, int flag);
 int Xorriso_option_check_md5(struct XorrisO *xorriso,
                              int argc, char **argv, int *idx, int flag);
 
-/* Option -check_media */
+/* Command -check_media */
 int Xorriso_option_check_media(struct XorrisO *xorriso,
                                int argc, char **argv, int *idx, int flag);
 
-/* Option -check_media_defaults */
+/* Command -check_media_defaults */
 int Xorriso_option_check_media_defaults(struct XorrisO *xorriso,
                                     int argc, char **argv, int *idx, int flag);
 
-/* Option -chgrp alias -chgrpi , chgrp_r alias chgrpi */
+/* Command -chgrp alias -chgrpi , chgrp_r alias chgrpi */
 /* @param flag bit0=recursive (-chgrp_r)
 */
 int Xorriso_option_chgrpi(struct XorrisO *xorriso, char *gid,
                           int argc, char **argv, int *idx, int flag);
 
-/* Option -chmod alias -chmodi , -chmod_r alias chmod_ri */
+/* Command -chmod alias -chmodi , -chmod_r alias chmod_ri */
 /* @param flag bit0=recursive (-chmod_r)
 */
 int Xorriso_option_chmodi(struct XorrisO *xorriso, char *mode,
                           int argc, char **argv, int *idx, int flag);
 
-/* Option -chown alias -chowni , chown_r alias chown_ri */
+/* Command -chown alias -chowni , chown_r alias chown_ri */
 /* @param flag bit0=recursive (-chown_r)
 */
 int Xorriso_option_chowni(struct XorrisO *xorriso, char *uid,
                           int argc, char **argv, int *idx, int flag);
 
-/* Option -clone */
+/* Command -clone */
 /* @since 1.0.2 */
 int Xorriso_option_clone(struct XorrisO *xorriso, char *origin, char *dest,
                          int flag);
 
-/* Option -close "on"|"off" */
+/* Command -close "on"|"off" */
 int Xorriso_option_close(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -close_damaged */
+/* Command -close_damaged */
 /* @since 1.1.0 */
 int Xorriso_option_close_damaged(struct XorrisO *xorriso, char *mode,
                                  int flag);
 
-/* Option -close_filter_list */
+/* Command -close_filter_list */
 int Xorriso_option_close_filter_list(struct XorrisO *xorriso, int flag);
 
-/* Option -commit */
-/* @param flag bit0= leave indrive and outdrive aquired as they were,
-                     i.e. do not aquire outdrive as new in-out-drive
+/* Command -commit */
+/* @param flag bit0= leave indrive and outdrive acquired as they were,
+                     i.e. do not acquire outdrive as new in-out-drive
                bit1= do not perform eventual -reassure
    @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_commit(struct XorrisO *xorriso, int flag);
 
-/* Option -commit_eject  */
+/* Command -commit_eject  */
 /* @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_commit_eject(struct XorrisO *xorriso, char *which, int flag);
 
-/* Option -compare and -compare_r
+/* Command -compare and -compare_r
    @param flag bit0= issue summary message
                bit1= do not reset pacifier, no final pacifier message
                bit2= do not issue pacifier messages at all
@@ -757,99 +784,104 @@ int Xorriso_option_commit_eject(struct XorrisO *xorriso, char *which, int flag);
 int Xorriso_option_compare(struct XorrisO *xorriso, char *disk_path,
                            char *iso_path, int flag);
 
-/* Option -compliance */
+/* Command -compliance */
 int Xorriso_option_compliance(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -copyright_file */
+/* Command -copyright_file */
 /* @since 0.6.0 */
 int Xorriso_option_copyright_file(struct XorrisO *xorriso, char *name,
                                   int flag);
 
-/* Option -cp_clone */
+/* Command -cp_clone */
 /* @since 1.0.2 */
 int Xorriso_option_cp_clone(struct XorrisO *xorriso, int argc, char **argv,
                             int *idx, int flag);
 
-/* Option -cpr alias -cpri */
+/* Command -cpr alias -cpri */
 int Xorriso_option_cpri( struct XorrisO *xorriso, int argc, char **argv,
                          int *idx, int flag);
 
-/* Options -cpx , -cpax, -cp_rx , -cp_rax */
+/* Command -cpx , -cpax, -cp_rx , -cp_rax */
 /* @param flag bit0= recursive (-cp_rx, -cp_rax)
                bit1= full property restore (-cpax, -cp_rax)
 */
 int Xorriso_option_cpx(struct XorrisO *xorriso, int argc, char **argv,
                          int *idx, int flag);
 
-/* Option -cut_out */
+/* Command -cut_out */
 int Xorriso_option_cut_out(struct XorrisO *xorriso, char *disk_path,
                 char *start, char *count, char *iso_rr_path, int flag);
 
-/* Options -dev , -indev, -outdev */
+/* Command -dev , -indev, -outdev */
 /* @param flag bit0=use as indev , bit1= use as outdev
    @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_dev(struct XorrisO *xorriso, char *adr, int flag);
 
-/* Option -devices */
+/* Command -data_cache_size */
+/* @since 1.2.2 */
+int Xorriso_option_data_cache_size(struct XorrisO *xorriso, char *num_tiles,
+                                   char *tile_blocks, int flag);
+
+/* Command -devices */
 /* @param flag bit0= perform -device_links rather than -devices
    @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_devices(struct XorrisO *xorriso, int flag);
 
-/* Option -dialog "on"|"off" */
+/* Command -dialog "on"|"off" */
 int Xorriso_option_dialog(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -disk_dev_ino "on"|"off" */
+/* Command -disk_dev_ino "on"|"off" */
 int Xorriso_option_disk_dev_ino(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -disk_pattern "on"|"ls"|"off" */
+/* Command -disk_pattern "on"|"ls"|"off" */
 int Xorriso_option_disk_pattern(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -displacement [-]offset */
+/* Command -displacement [-]offset */
 /* @since 0.6.6 */
 int Xorriso_option_displacement(struct XorrisO *xorriso, char *value,
                                 int flag);
 
-/* Option -drive_class */
+/* Command -drive_class */
 int Xorriso_option_drive_class(struct XorrisO *xorriso,
                                char *d_class, char *pattern, int flag);
 
-/* Option -dummy "on"|"off" */
+/* Command -dummy "on"|"off" */
 int Xorriso_option_dummy(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -dvd_obs "default"|"32k"|"64k" */
+/* Command -dvd_obs "default"|"32k"|"64k" */
 int Xorriso_option_dvd_obs(struct XorrisO *xorriso, char *obs, int flag);
 
-/* Option -early_stdio_test */
+/* Command -early_stdio_test */
 /* @since 1.0.6 */
 int Xorriso_option_early_stdio_test(struct XorrisO *xorriso, char *mode,
                                     int flag);
 
-/* Option -eject */
+/* Command -eject */
 /* @param flag bit0=do not report toc of eventually remaining drives
 */
 int Xorriso_option_eject(struct XorrisO *xorriso, char *which, int flag);
 
-/* Options -end , and -rollback_end */
+/* Command -end , and -rollback_end */
 /* @param flag bit0= discard pending changes 
    @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_end(struct XorrisO *xorriso, int flag);
 
-/* Option -errfile_log marked|plain  path|-|"" */
+/* Command -errfile_log marked|plain  path|-|"" */
 int Xorriso_option_errfile_log(struct XorrisO *xorriso,
                                char *mode, char *path, int flag);
 
-/* Option -error_behavior */
+/* Command -error_behavior */
 int Xorriso_option_error_behavior(struct XorrisO *xorriso, 
                                   char *occasion, char *behavior, int flag);
 
-/* Option -external_filter */
+/* Command -external_filter */
 int Xorriso_option_external_filter(struct XorrisO *xorriso,
                                    int argc, char **argv, int *idx, int flag);
 
-/* Options -extract , -extract_single */
+/* Command -extract , -extract_single */
 /* @param flag bit0=do not report the restored item
                bit1=do not reset pacifier, no final pacifier message
                bit5= -extract_single: do not insert directory tree
@@ -857,16 +889,16 @@ int Xorriso_option_external_filter(struct XorrisO *xorriso,
 int Xorriso_option_extract(struct XorrisO *xorriso, char *disk_path,
                        char *iso_path, int flag);
 
-/* Option -extract_cut */
+/* Command -extract_cut */
 int Xorriso_option_extract_cut(struct XorrisO *xorriso, char *iso_rr_path,
                           char *start, char *count, char *disk_path, int flag);
 
-/* Option -file_size_limit */
+/* Command -file_size_limit */
 int Xorriso_option_file_size_limit(struct XorrisO *xorriso,
                                    int argc, char **argv, int *idx, int flag);
 
 
-/* Option -find alias -findi, and -findx */
+/* Command -find alias -findi, and -findx */
 /* @param flag bit0= -findx rather than -findi
                bit1= do not reset pacifier, no final pacifier message
                      do not reset find_compare_result
@@ -874,79 +906,85 @@ int Xorriso_option_file_size_limit(struct XorrisO *xorriso,
 int Xorriso_option_find(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag);
 
-/* Option -follow */
+/* Command -follow */
 int Xorriso_option_follow(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -fs */
+/* Command -fs */
 int Xorriso_option_fs(struct XorrisO *xorriso, char *size, int flag);
 
-/* Option -getfacl alias -getfacli, -getfacl_r alias -getfacl_ri */
+/* Command -getfacl alias -getfacli, -getfacl_r alias -getfacl_ri */
 /* @param flag bit0=recursive -getfacl_r
 */
 int Xorriso_option_getfacli(struct XorrisO *xorriso,
                             int argc, char **argv, int *idx, int flag);
 
-/* Option -gid */
+/* Command -gid */
 int Xorriso_option_gid(struct XorrisO *xorriso, char *gid, int flag);
 
-/* Option -grow_blindly */
+/* Command -grow_blindly */
 int Xorriso_option_grow_blindly(struct XorrisO *xorriso, char *msc2, int flag);
 
-/* Option -hardlinks "on"|"off" */
+/* Command -hardlinks "on"|"off" */
 int Xorriso_option_hardlinks(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -help and part of -prog_help */
+/* Command -help and part of -prog_help */
 int Xorriso_option_help(struct XorrisO *xorriso, int flag);
 
-/* Option -hide */
+/* Command -hide */
 /* @since 0.6.0 */
 int Xorriso_option_hide(struct XorrisO *xorriso, char *hide_state,
                         int argc, char **argv, int *idx, int flag);
 
-/* Option -history */
+/* Command -history */
 int Xorriso_option_history(struct XorrisO *xorriso, char *line, int flag);
 
-/* Option -iso_rr_pattern "on"|"ls"|"off" */
+/* Command -iso_rr_pattern "on"|"ls"|"off" */
 int Xorriso_option_iso_rr_pattern(struct XorrisO *xorriso, char *mode,
                                   int flag);
 
-/* Option -jigdo aspect argument */
+/* Command -jigdo aspect argument */
 /* @since 0.6.4 */
 int Xorriso_option_jigdo(struct XorrisO *xorriso, char *aspect, char *arg,
                          int flag);
 
-/* Option -joliet "on"|"off" */
+/* Command -joliet "on"|"off" */
 int Xorriso_option_joliet(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -list_delimiter */
+/* Command -list_arg_sorting */
+/* @since 1.2.2 */
+int Xorriso_option_list_arg_sorting(struct XorrisO *xorriso, int flag);
+
+/* Command -list_delimiter */
 int Xorriso_option_list_delimiter(struct XorrisO *xorriso, char *text,
                                   int flag);
 
-/* Option -list_extras */
+/* Command -list_extras */
+/* @since 1.1.6 */
 int Xorriso_option_list_extras(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -list_formats */
+/* Command -list_formats */
 int Xorriso_option_list_formats(struct XorrisO *xorriso, int flag);
 
-/* Option -list_profiles */
+/* Command -list_profiles */
 int Xorriso_option_list_profiles(struct XorrisO *xorriso, char *which,
                                  int flag);
 
-/* Option -list_speeds */
+/* Command -list_speeds */
+/* @since 1.1.2 */
 int Xorriso_option_list_speeds(struct XorrisO *xorriso, int flag);
 
-/* Option -load session|track|sbsector value */
+/* Command -load session|track|sbsector value */
 /* @param flag bit0= with adr_mode sbsector: adr_value is possibly 16 too high
    @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_load(struct XorrisO *xorriso, char *adr_mode,
                           char *adr_value, int flag);
 
-/* Option -logfile */
+/* Command -logfile */
 int Xorriso_option_logfile(struct XorrisO *xorriso, char *channel, 
                                                       char *fileadr, int flag);
 
-/* Options -ls  alias -lsi   and -lsl  alias -lsli
+/* Command -ls  alias -lsi   and -lsl  alias -lsli
        and -lsd alias -lsdi  and -lsdl alias -lsdli
        and -du  alias -dui   and -dus  alias -dusi
    @param flag bit0= long format (-lsl , -du)
@@ -957,7 +995,7 @@ int Xorriso_option_logfile(struct XorrisO *xorriso, char *channel,
 int Xorriso_option_lsi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
 
-/* Options -lsx, -lslx, -lsdx , -lsdlx , -dux , -dusx
+/* Command -lsx, -lslx, -lsdx , -lsdlx , -dux , -dusx
    @param flag bit0= long format (-lslx , -dux)
                bit1= do not expand patterns but use literally
                bit2= du rather than ls
@@ -966,14 +1004,14 @@ int Xorriso_option_lsi(struct XorrisO *xorriso, int argc, char **argv,
 int Xorriso_option_lsx(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
 
-/* Option -map */
+/* Command -map */
 /* @param flag bit0=do not report the added item
                bit1=do not reset pacifier, no final pacifier message
 */
 int Xorriso_option_map(struct XorrisO *xorriso, char *disk_path,
                        char *iso_path, int flag);
 
-/* Options -map_l , -compare_l , -update_l , -extract_l */
+/* Command -map_l , -compare_l , -update_l , -extract_l */
 /* @param flag bit8-11= mode 0= -map_l
                              1= -compare_l
                              2= -update_l
@@ -982,17 +1020,17 @@ int Xorriso_option_map(struct XorrisO *xorriso, char *disk_path,
 int Xorriso_option_map_l(struct XorrisO *xorriso, int argc, char **argv,
                          int *idx, int flag);
 
-/* Option -mark */
+/* Command -mark */
 int Xorriso_option_mark(struct XorrisO *xorriso, char *mark, int flag);
 
-/* Option -md5 */
+/* Command -md5 */
 int Xorriso_option_md5(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -mkdir alias -mkdiri */
+/* Command -mkdir alias -mkdiri */
 int Xorriso_option_mkdiri(struct XorrisO *xorriso, int argc, char **argv,
                           int *idx, int flag);
 
-/* Options -mount , -mount_cmd , -session_string */
+/* Command -mount , -mount_cmd , -session_string */
 /* @param bit0= -mount_cmd: print mount command to result channel rather
                             than performing it
           bit1= perform -session_string rather than -mount_cmd
@@ -1000,17 +1038,17 @@ int Xorriso_option_mkdiri(struct XorrisO *xorriso, int argc, char **argv,
 int Xorriso_option_mount(struct XorrisO *xorriso, char *dev, char *adr_mode,
                          char *adr, char *cmd, int flag);
 
-/* Option -mount_opts option[:...] */
+/* Command -mount_opts option[:...] */
 int Xorriso_option_mount_opts(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -mv alias -mvi */
+/* Command -mv alias -mvi */
 int Xorriso_option_mvi(struct XorrisO *xorriso, int argc, char **argv,
                       int *idx, int flag);
 
-/* Option -no_rc */
+/* Command -no_rc */
 int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag);
 
-/* Option -not_leaf , -as mkisofs -hide without '/' */
+/* Command -not_leaf , -as mkisofs -hide without '/' */
 /* @param flag  bit0= add to iso_rr hide list rather than to disk exclusions
                       @since 0.6.0
                 bit1= add to joliet hide list rather than disk exclusions
@@ -1018,14 +1056,14 @@ int Xorriso_option_no_rc(struct XorrisO *xorriso, int flag);
 */
 int Xorriso_option_not_leaf(struct XorrisO *xorriso, char *pattern, int flag);
 
-/* Option -not_list , -quoted_not_list */
+/* Command -not_list , -quoted_not_list */
 /* @param flag bit0= -quoted_not_list */
 int Xorriso_option_not_list(struct XorrisO *xorriso, char *adr, int flag);
 
-/* Option -not_mgt */
+/* Command -not_mgt */
 int Xorriso_option_not_mgt(struct XorrisO *xorriso, char *setting, int flag);
 
-/* Option -not_paths , -as mkisofs -hide with '/' */
+/* Command -not_paths , -as mkisofs -hide with '/' */
 /* @param flag  bit0= add to iso_rr hide list rather than to disk exclusions
                       @since 0.6.0
                 bit1= add to joliet hide list rather than disk exclusions
@@ -1035,45 +1073,45 @@ int Xorriso_option_not_mgt(struct XorrisO *xorriso, char *setting, int flag);
 int Xorriso_option_not_paths(struct XorrisO *xorriso, int argc, char **argv,
                              int *idx, int flag);
 
-/* Option -options_from_file */
+/* Command -options_from_file */
 /* @return <=0 error , 1 = success , 3 = request to end program run */
 int Xorriso_option_options_from_file(struct XorrisO *xorriso, char *adr,
                                      int flag);
 
-/* Option -osirrox "on"|"off" */
+/* Command -osirrox "on"|"off" */
 int Xorriso_option_osirrox(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -overwrite "on"|"nondir"|"off" */
+/* Command -overwrite "on"|"nondir"|"off" */
 int Xorriso_option_overwrite(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -pacifier */
+/* Command -pacifier */
 int Xorriso_option_pacifier(struct XorrisO *xorriso, char *style, int flag);
 
-/* Option -padding */
+/* Command -padding */
 int Xorriso_option_padding(struct XorrisO *xorriso, char *size, int flag);
 
-/* Option -page */
+/* Command -page */
 int Xorriso_option_page(struct XorrisO *xorriso, int len, int width, int flag);
 
-/* Option -paste_in */
+/* Command -paste_in */
 int Xorriso_option_paste_in(struct XorrisO *xorriso, char *iso_rr_path,
                           char *disk_path, char *start, char *count, int flag);
 
-/* Option -path_list , -quoted_path_list */
+/* Command -path_list , -quoted_path_list */
 /* @param flag bit0= -quoted_path_list */
 int Xorriso_option_path_list(struct XorrisO *xorriso, char *adr, int flag);
 
-/* Option -pathspecs */
+/* Command -pathspecs */
 int Xorriso_option_pathspecs(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -pkt_output */
+/* Command -pkt_output */
 int Xorriso_option_pkt_output(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -preparer_id */
+/* Command -preparer_id */
 /* @since 0.6.2 */
 int Xorriso_option_preparer_id(struct XorrisO *xorriso, char *name, int flag);
 
-/* Options -print, -print_info , -print_mark */
+/* Command -print, -print_info , -print_mark */
 /* @param flag bit0-1= output channel:
                        0= result channel
                        1= info channel @since 1.0.6
@@ -1081,147 +1119,152 @@ int Xorriso_option_preparer_id(struct XorrisO *xorriso, char *name, int flag);
 */
 int Xorriso_option_print(struct XorrisO *xorriso, char *text, int flag);
 
-/* Option -print_size
+/* Command -print_size
    @param flag bit0= report in mkisofs compatible form on real stdout
 */
 int Xorriso_option_print_size(struct XorrisO *xorriso, int flag);
 
-/* Option -prog */
+/* Command -prog */
 int Xorriso_option_prog(struct XorrisO *xorriso, char *name, int flag);
 
-/* Option -prompt */
+/* Command -prompt */
 int Xorriso_option_prompt(struct XorrisO *xorriso, char *text, int flag);
 
-/* Option -prog_help */
+/* Command -prog_help */
 int Xorriso_option_prog_help(struct XorrisO *xorriso, char *name, int flag);
 
-/* Option -publisher */
+/* Command -publisher */
 int Xorriso_option_publisher(struct XorrisO *xorriso, char *name, int flag);
 
-/* Option -pvd_info */
+/* Command -pvd_info */
 int Xorriso_option_pvd_info(struct XorrisO *xorriso, int flag);
 
-/* Option -pwd alias -pwdi */
+/* Command -pwd alias -pwdi */
 int Xorriso_option_pwdi(struct XorrisO *xorriso, int flag);
 
-/* Option -pwdx */
+/* Command -pwdx */
 int Xorriso_option_pwdx(struct XorrisO *xorriso, int flag);
 
-/* Option -read_mkisofsrc */
+/* Command -read_mkisofsrc */
 /* @since 0.6.0 */
 int Xorriso_option_read_mkisofsrc(struct XorrisO *xorriso, int flag);
 
-/* Option -reassure "on"|"tree"|"off" */
+/* Command -reassure "on"|"tree"|"off" */
 int Xorriso_option_reassure(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -report_about */
+/* Command -report_about */
 int Xorriso_option_report_about(struct XorrisO *xorriso, char *severity, 
                                 int flag);
 
-/* Option -return_with */
+/* Command -return_with */
 int Xorriso_option_return_with(struct XorrisO *xorriso, char *severity,
                                int exit_value, int flag);
 
-/* Options -rm alias -rmi , -rm_r alias -rm_ri , -rmdir alias -rmdiri */
+/* Command -rm alias -rmi , -rm_r alias -rm_ri , -rmdir alias -rmdiri */
 /* @param flag bit0=recursive , bit2= remove empty directory: rmdir */
 int Xorriso_option_rmi(struct XorrisO *xorriso, int argc, char **argv,
                        int *idx, int flag);
 
-/* Option -rollback */
+/* Command -rollback */
 /* @param flag bit0= do not -reassure
    @return <=0 error , 1 success, 2 revoked by -reassure
 */
 int Xorriso_option_rollback(struct XorrisO *xorriso, int flag);
 
-/* Option -rom_toc_scan */
-int Xorriso_option_rom_toc_scan(struct XorrisO *xorriso, char *mode,
-                                int flag);
+/* Command -rom_toc_scan */
+int Xorriso_option_rom_toc_scan(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -scdbackup_tag */
+/* Command -rr_reloc_dir */
+/* @since 1.2.2 */
+int Xorriso_option_rr_reloc_dir(struct XorrisO *xorriso, char *name, int flag);
+
+/* Command -scdbackup_tag */
 int Xorriso_option_scdbackup_tag(struct XorrisO *xorriso, char *list_path,
                                  char *record_name, int flag);
-/* Option -scsi_log */
+/* Command -scsi_log */
 int Xorriso_option_scsi_log(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -session_log */
+/* Command -session_log */
 int Xorriso_option_session_log(struct XorrisO *xorriso, char *path, int flag);
 
-/* Option -setfacl_list alias -setfacl_listi */
+/* Command -setfacl_list alias -setfacl_listi */
 int Xorriso_option_setfacl_listi(struct XorrisO *xorriso, char *disk_path,
                                  int flag);
 
-/* Option -setfacl alias -setfacli , -setfacl_r  alias -setfacl_ri */
+/* Command -setfacl alias -setfacli , -setfacl_r  alias -setfacl_ri */
 /* @param flag bit0=recursive -setfacl_r
 */
 int Xorriso_option_setfacli(struct XorrisO *xorriso, char *acl_text,
                             int argc, char **argv, int *idx, int flag);
 
-/* Options -setfattr alias -setfattri, -setfattr_r alias -setfattr_ri */
+/* Command -setfattr alias -setfattri, -setfattr_r alias -setfattr_ri */
 /* @param flag   bit0=recursive -setfattr_r
 */
 int Xorriso_option_setfattri(struct XorrisO *xorriso, char *name, char *value,
                             int argc, char **argv, int *idx, int flag);
 
-/* Option -setfattr_list alias -setfattr_listi */
+/* Command -setfattr_list alias -setfattr_listi */
 int Xorriso_option_setfattr_listi(struct XorrisO *xorriso, char *path, 
                                   int flag);
 
-/* Options -set_filter , -set_filter_r */ 
+/* Command -set_filter , -set_filter_r */ 
 /* @param flag   bit0=recursive -set_filter_r
 */
 int Xorriso_option_set_filter(struct XorrisO *xorriso, char *name,
                               int argc, char **argv, int *idx, int flag);
 
-/* Option -signal_handling */
+/* Command -signal_handling */
 /* @param flag bit0= do not yet install the eventual handler
+   @since 1.1.0
 */
 int Xorriso_option_signal_handling(struct XorrisO *xorriso, char *mode,
                                    int flag);
 
-/* Option -sleep */
+/* Command -sleep */
+/* @since 1.1.8 */
 int Xorriso_option_sleep(struct XorrisO *xorriso, char *duration, int flag);
 
-/* Option -speed */
+/* Command -speed */
 int Xorriso_option_speed(struct XorrisO *xorriso, char *speed, int flag);
 
-/* Option -split_size */
+/* Command -split_size */
 int Xorriso_option_split_size(struct XorrisO *xorriso, char *s, int flag);
 
-/* Option -status */
+/* Command -status */
 int Xorriso_option_status(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -status_history_max */
+/* Command -status_history_max */
 int Xorriso_option_status_history_max(struct XorrisO *xorriso, int num1, 
                                       int flag);
 
-/* Option -stdio_sync "on"|"off"|size */
+/* Command -stdio_sync "on"|"off"|size */
 int Xorriso_option_stdio_sync(struct XorrisO *xorriso, char *rythm, int flag);
 
-/* Option -stream_recording */
+/* Command -stream_recording */
 int Xorriso_option_stream_recording(struct XorrisO *xorriso, char *mode,
                                     int flag);
 
-/* Option -system_id */
+/* Command -system_id */
 int Xorriso_option_system_id(struct XorrisO *xorriso, char *name, int flag);
 
-/* Option -tell_media_space */
+/* Command -tell_media_space */
 int Xorriso_option_tell_media_space(struct XorrisO *xorriso, int flag);
 
-/* Option -temp_mem_limit */
+/* Command -temp_mem_limit */
 int Xorriso_option_temp_mem_limit(struct XorrisO *xorriso, char *size, 
                                   int flag);
 
-/* Option -toc */
+/* Command -toc */
 int Xorriso_option_toc(struct XorrisO *xorriso, int flag);
 
-/* Option -uid */
+/* Command -uid */
 int Xorriso_option_uid(struct XorrisO *xorriso, char *uid, int flag);
 
-/* Option -unregister_filter */
+/* Command -unregister_filter */
 int Xorriso_option_unregister_filter(struct XorrisO *xorriso, char *name,
                                      int flag);
 
-/* Options -update and -update_r
+/* Command -update and -update_r
    @param flag bit0= issue summary message
                bit1= do not reset pacifier, no final pacifier message
                bit2= do not issue pacifier messages at all
@@ -1230,28 +1273,33 @@ int Xorriso_option_unregister_filter(struct XorrisO *xorriso, char *name,
 int Xorriso_option_update(struct XorrisO *xorriso, char *disk_path,
                           char *iso_path, int flag);
 
-/* Option -use_readline */
+/* Command -use_readline */
 int Xorriso_option_use_readline(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -version */
+/* Command -version */
 int Xorriso_option_version(struct XorrisO *xorriso, int flag);
 
-/* Option -volid */
+/* Command -volid */
 /* @param flag bit0= do not warn of problematic volid
 */
 int Xorriso_option_volid(struct XorrisO *xorriso, char *volid, int flag);
 
-/* Option -volset_id */
+/* Command -volset_id */
 int Xorriso_option_volset_id(struct XorrisO *xorriso, char *name, int flag);
 
-/* Option -volume_date */
+/* Command -volume_date */
 int Xorriso_option_volume_date(struct XorrisO *xorriso,
                                char *time_type, char *timestring, int flag);
 
-/* Option -xattr "on"|"off" */
+/* There is no Xorriso_option_x() because -x has an effect only in
+   Xorriso_prescan_args(). Use the flag bits of Xorriso_interpreter() if
+   you want to impose command sorting on your own.
+*/
+
+/* Command -xattr "on"|"off" */
 int Xorriso_option_xattr(struct XorrisO *xorriso, char *mode, int flag);
 
-/* Option -zisofs */
+/* Command -zisofs */
 int Xorriso_option_zisofs(struct XorrisO *xorriso, char *mode, int flag);
 
 
