@@ -2,7 +2,7 @@
 /* Command line oriented batch and dialog tool which creates, loads,
    manipulates and burns ISO 9660 filesystem images.
 
-   Copyright 2007-2011 Thomas Schmitt, <scdbackup@gmx.net>
+   Copyright 2007-2012 Thomas Schmitt, <scdbackup@gmx.net>
 
    Provided under GPL version 2 or later.
 
@@ -102,6 +102,8 @@ struct XorrisO { /* the global context of xorriso */
  char rc_filenames[Xorriso_rc_nuM][SfileadrL];
  int rc_filename_count;
 
+ int arrange_args;
+
  /* Whether .mkisofsrc has already been read */
  int mkisofsrc_done;
 
@@ -168,6 +170,8 @@ struct XorrisO { /* the global context of xorriso */
  char scdbackup_tag_listname[SfileadrL];
 
  int relax_compliance;        /* opaque bitfield to be set by xorrisoburn */
+ char rr_reloc_dir[256];
+ int rr_reloc_flags;
  int untranslated_name_len;
  int do_follow_pattern;
  int do_follow_param;
@@ -263,6 +267,10 @@ struct XorrisO { /* the global context of xorriso */
                                   burn_write_opts_set_start_byte().
                           */
 
+ int cache_num_tiles;     /* -data_cache_size */
+ int cache_tile_blocks;
+ int cache_default;       /* bit0= cache_num_tiles, bit1= cache_tile_blocks */
+
  int do_calm_drive;      /* bit0= calm down drive after aquiring it */
 
  char indev[SfileadrL];
