@@ -19,7 +19,7 @@
 */
 #define xorriso_libisoburn_req_major  1
 #define xorriso_libisoburn_req_minor  2
-#define xorriso_libisoburn_req_micro  0
+#define xorriso_libisoburn_req_micro  2
 
 
 struct SpotlisT;          /* List of intervals with different read qualities */
@@ -595,5 +595,8 @@ int Xorriso_close_damaged(struct XorrisO *xorriso, int flag);
 
 int Xorriso_list_extras(struct XorrisO *xorriso, char *mode, int flag);
 
+int Xorriso_set_data_cache(struct XorrisO *xorriso, void *ropts,
+                           int num_tiles, int tile_blocks, int flag);
+
 #endif /* Xorrisoburn_includeD */
 
